searchData={"items":[{"type":"module","title":"KafkaEx","doc":"Kafka API\n\nThis module is the main API for users of the KafkaEx library.\n\nMost of these functions either use the default worker (registered as\n`:kafka_ex`) by default or can take a registered name or pid via a\n`worker_name` option.\n\n```\n# create an unnamed worker\n{:ok, pid} = KafkaEx.create_worker(:no_name)\n\nKafkaEx.fetch(\"some_topic\", 0, worker_name: pid)\n```","ref":"KafkaEx.html"},{"type":"function","title":"KafkaEx.api_versions/1","doc":"Retrieve supported api versions for each api key.","ref":"KafkaEx.html#api_versions/1"},{"type":"function","title":"KafkaEx.build_worker_options/1","doc":"Builds options to be used with workers\n\nMerges the given options with defaults from the application env config.\nReturns `{:error, :invalid_consumer_options}` if the consumer group\nconfiguration is invalid, and `{:ok, merged_options}` otherwise.\n\nNote this happens automatically when using `KafkaEx.create_worker`.","ref":"KafkaEx.html#build_worker_options/1"},{"type":"function","title":"KafkaEx.consumer_group/1","doc":"Returns the name of the consumer group for the given worker.\n\nWorker may be an atom or pid.  The default worker is used by default.","ref":"KafkaEx.html#consumer_group/1"},{"type":"function","title":"KafkaEx.consumer_group_metadata/2","doc":"","ref":"KafkaEx.html#consumer_group_metadata/2"},{"type":"function","title":"KafkaEx.create_topics/2","doc":"Create topics. Must provide a list of CreateTopicsRequest, each containing\nall the information needed for the creation of a new topic.\n\nSee available topic configuration options at https://docs.confluent.io/platform/current/installation/configuration/topic-configs.html","ref":"KafkaEx.html#create_topics/2"},{"type":"function","title":"Example - KafkaEx.create_topics/2","doc":"```elixir\niex> KafkaEx.start_link_worker(:my_kafka_worker)\n{:ok, #PID<0.659.0>}\niex> KafkaEx.create_topics(\n...>  [\n...>   %KafkaEx.Protocol.CreateTopics.TopicRequest{\n...>     topic: \"my_topic_name\",\n...>      num_partitions: 1,\n...>      replication_factor: 1,\n...>      replica_assignment: [],\n...>      config_entries: [\n...>        %{config_name: \"cleanup.policy\", config_value: \"delete\"},\n...>        %{config_name: \"delete.retention.ms\", config_value: \"864000000\"} # 1 day\n...>      ]\n...>    }\n...>  ],\n...>  timeout: 10_000,\n...>  worker_name: :my_kafka_worker\n...> )\n%KafkaEx.Protocol.CreateTopics.Response{\n  topic_errors: [\n    %KafkaEx.Protocol.CreateTopics.TopicError{\n      error_code: :no_error,\n      topic_name: \"my_topic_name\"\n    }\n  ]\n}\n```","ref":"KafkaEx.html#create_topics/2-example"},{"type":"function","title":"KafkaEx.create_worker/2","doc":"create_worker creates KafkaEx workers\n\nOptional arguments(KeywordList)\n- consumer_group: Name of the group of consumers, `:no_consumer_group` should be passed for Kafka < 0.8.2, defaults to `Application.get_env(:kafka_ex, :consumer_group)`\n- uris: List of brokers in `{\"host\", port}` or comma separated value `\"host:port,host:port\"` form, defaults to `Application.get_env(:kafka_ex, :brokers)`\n- metadata_update_interval: How often `kafka_ex` would update the Kafka cluster metadata information in milliseconds, default is 30000\n- consumer_group_update_interval: How often `kafka_ex` would update the Kafka cluster consumer_groups information in milliseconds, default is 30000\n- use_ssl: Boolean flag specifying if ssl should be used for the connection by the worker to Kafka, default is false\n- ssl_options: see SSL OPTION DESCRIPTIONS - CLIENT SIDE at http://erlang.org/doc/man/ssl.html, default is []\n\nReturns `{:error, error_description}` on invalid arguments","ref":"KafkaEx.html#create_worker/2"},{"type":"function","title":"Example - KafkaEx.create_worker/2","doc":"```elixir\niex> KafkaEx.create_worker(:pr) # where :pr is the name of the worker created\n{:ok, #PID<0.171.0>}\niex> KafkaEx.create_worker(:pr, uris: [{\"localhost\", 9092}])\n{:ok, #PID<0.172.0>}\niex> KafkaEx.create_worker(:pr, [uris: [{\"localhost\", 9092}], consumer_group: \"foo\"])\n{:ok, #PID<0.173.0>}\niex> KafkaEx.create_worker(:pr, consumer_group: nil)\n{:error, :invalid_consumer_group}\n```","ref":"KafkaEx.html#create_worker/2-example"},{"type":"function","title":"KafkaEx.delete_topics/2","doc":"Delete topics. Must provide a list of topic names.","ref":"KafkaEx.html#delete_topics/2"},{"type":"function","title":"Example - KafkaEx.delete_topics/2","doc":"```elixir\niex> KafkaEx.delete_topics([\"my_topic_name\"], [])\n%KafkaEx.Protocol.DeleteTopics.Response{\n  topic_errors: [\n    %KafkaEx.Protocol.DeleteTopics.TopicError{\n      error_code: :no_error,\n      topic_name: \"my_topic_name\"\n    }\n  ]\n}\n```","ref":"KafkaEx.html#delete_topics/2-example"},{"type":"function","title":"KafkaEx.describe_group/2","doc":"Sends a request to describe a group identified by its name.\nWe support only one consumer group per request for now, as we don't\ngroup requests by group coordinator.\nThis is a new client implementation, and is not compatible with the old clients","ref":"KafkaEx.html#describe_group/2"},{"type":"function","title":"KafkaEx.earliest_offset/3","doc":"Get the offset of the earliest message still persistent in Kafka","ref":"KafkaEx.html#earliest_offset/3"},{"type":"function","title":"Example - KafkaEx.earliest_offset/3","doc":"```elixir\niex> KafkaEx.earliest_offset(\"foo\", 0)\n[%KafkaEx.Protocol.Offset.Response{partition_offsets: [%{error_code: 0, offset: [0], partition: 0}], topic: \"foo\"}]\n```","ref":"KafkaEx.html#earliest_offset/3-example"},{"type":"function","title":"KafkaEx.fetch/3","doc":"Fetch a set of messages from Kafka from the given topic and partition ID\n\nOptional arguments(KeywordList)\n- offset: When supplied the fetch would start from this offset, otherwise would start from the last committed offset of the consumer_group the worker belongs to. For Kafka < 0.8.2 you should explicitly specify this.\n- worker_name: the worker we want to run this fetch request through. Default is :kafka_ex\n- wait_time: maximum amount of time in milliseconds to block waiting if insufficient data is available at the time the request is issued. Default is 10\n- min_bytes: minimum number of bytes of messages that must be available to give a response. If the client sets this to 0 the server will always respond immediately, however if there is no new data since their last request they will just get back empty message sets. If this is set to 1, the server will respond as soon as at least one partition has at least 1 byte of data or the specified timeout occurs. By setting higher values in combination with the timeout the consumer can tune for throughput and trade a little additional latency for reading only large chunks of data (e.g. setting wait_time to 100 and setting min_bytes 64000 would allow the server to wait up to 100ms to try to accumulate 64k of data before responding). Default is 1\n- max_bytes: maximum bytes to include in the message set for this partition. This helps bound the size of the response. Default is 1,000,000\n- auto_commit: specifies if the last offset should be committed or not. Default is true. You must set this to false when using Kafka < 0.8.2 or `:no_consumer_group`.\n- api_version: Version of the Fetch API message to send (Kayrock client only, default: 0)\n- offset_commit_api_version: Version of the OffsetCommit API message to send\n  (Kayrock client only, only relevant for auto commit, default: 0, use 2+ to\n  store offsets in Kafka instead of Zookeeper)","ref":"KafkaEx.html#fetch/3"},{"type":"function","title":"Example - KafkaEx.fetch/3","doc":"```elixir\niex> KafkaEx.fetch(\"foo\", 0, offset: 0)\n[\n  %KafkaEx.Protocol.Fetch.Response{partitions: [\n    %{error_code: 0, hw_mark_offset: 1, message_set: [\n      %{attributes: 0, crc: 748947812, key: nil, offset: 0, value: \"hey foo\"}\n    ], partition: 0}\n  ], topic: \"foo\"}\n]\n```","ref":"KafkaEx.html#fetch/3-example"},{"type":"function","title":"KafkaEx.heartbeat/2","doc":"Sends a heartbeat to maintain membership in a consumer group.","ref":"KafkaEx.html#heartbeat/2"},{"type":"function","title":"KafkaEx.join_group/2","doc":"Sends a request to join a consumer group.","ref":"KafkaEx.html#join_group/2"},{"type":"function","title":"KafkaEx.latest_offset/3","doc":"Get the offset of the latest message written to Kafka","ref":"KafkaEx.html#latest_offset/3"},{"type":"function","title":"Example - KafkaEx.latest_offset/3","doc":"```elixir\niex> KafkaEx.latest_offset(\"foo\", 0)\n[%KafkaEx.Protocol.Offset.Response{partition_offsets: [%{error_code: 0, offsets: [16], partition: 0}], topic: \"foo\"}]\n```","ref":"KafkaEx.html#latest_offset/3-example"},{"type":"function","title":"KafkaEx.leave_group/2","doc":"Sends a request to leave a consumer group.","ref":"KafkaEx.html#leave_group/2"},{"type":"function","title":"KafkaEx.metadata/1","doc":"Return metadata for the given topic; returns for all topics if topic is empty string\n\nOptional arguments(KeywordList)\n- worker_name: the worker we want to run this metadata request through, when none is provided the default worker `:kafka_ex` is used\n- topic: name of the topic for which metadata is requested, when none is provided all metadata is retrieved","ref":"KafkaEx.html#metadata/1"},{"type":"function","title":"Example - KafkaEx.metadata/1","doc":"```elixir\niex> KafkaEx.create_worker(:mt)\niex> KafkaEx.metadata(topic: \"foo\", worker_name: :mt)\n%KafkaEx.Protocol.Metadata.Response{brokers: [%KafkaEx.Protocol.Metadata.Broker{host: \"192.168.59.103\",\n   node_id: 49162, port: 49162, socket: nil}],\n topic_metadatas: [%KafkaEx.Protocol.Metadata.TopicMetadata{error_code: 0,\n   partition_metadatas: [%KafkaEx.Protocol.Metadata.PartitionMetadata{error_code: 0,\n     isrs: [49162], leader: 49162, partition_id: 0, replicas: [49162]}],\n   topic: \"foo\"}]}\n```","ref":"KafkaEx.html#metadata/1-example"},{"type":"function","title":"KafkaEx.offset/4","doc":"Get the offset of the message sent at the specified date/time","ref":"KafkaEx.html#offset/4"},{"type":"function","title":"Example - KafkaEx.offset/4","doc":"```elixir\niex> KafkaEx.offset(\"foo\", 0, {{2015, 3, 29}, {23, 56, 40}}) # Note that the time specified should match/be ahead of time on the server that kafka runs\n[%KafkaEx.Protocol.Offset.Response{partition_offsets: [%{error_code: 0, offset: [256], partition: 0}], topic: \"foo\"}]\n```","ref":"KafkaEx.html#offset/4-example"},{"type":"function","title":"KafkaEx.offset_commit/2","doc":"","ref":"KafkaEx.html#offset_commit/2"},{"type":"function","title":"KafkaEx.offset_fetch/2","doc":"","ref":"KafkaEx.html#offset_fetch/2"},{"type":"function","title":"KafkaEx.produce/2","doc":"Produces batch messages to kafka logs\n\nOptional arguments(KeywordList)\n- worker_name: the worker we want to run this metadata request through, when none is provided the default worker `:kafka_ex` is used","ref":"KafkaEx.html#produce/2"},{"type":"function","title":"Example - KafkaEx.produce/2","doc":"```elixir\niex> KafkaEx.produce(%KafkaEx.Protocol.Produce.Request{topic: \"foo\", partition: 0, required_acks: 1, messages: [%KafkaEx.Protocol.Produce.Message{value: \"hey\"}]})\n{:ok, 9772}\niex> KafkaEx.produce(%KafkaEx.Protocol.Produce.Request{topic: \"foo\", partition: 0, required_acks: 1, messages: [%KafkaEx.Protocol.Produce.Message{value: \"hey\"}]}, worker_name: :pr)\n{:ok, 9773}\n```","ref":"KafkaEx.html#produce/2-example"},{"type":"function","title":"KafkaEx.produce/4","doc":"Produces messages to Kafka logs (this is deprecated, use KafkaEx.produce/2 instead)\nOptional arguments(KeywordList)\n- worker_name: the worker we want to run this metadata request through, when none is provided the default worker `:kafka_ex` is used\n- key: is used for partition assignment, can be nil, when none is provided it is defaulted to nil\n- required_acks: indicates how many acknowledgements the servers should receive before responding to the request. If it is 0 the server will not send any response (this is the only case where the server will not reply to a request). If it is 1, the server will wait the data is written to the local log before sending a response. If it is -1 the server will block until the message is committed by all in sync replicas before sending a response. For any number > 1 the server will block waiting for this number of acknowledgements to occur (but the server will never wait for more acknowledgements than there are in-sync replicas), default is 0\n- timeout: provides a maximum time in milliseconds the server can await the receipt of the number of acknowledgements in RequiredAcks, default is 100 milliseconds\n- compression: specifies the compression type (:none, :snappy, :gzip)\n- api_version: Version of the Fetch API message to send (Kayrock client only, default: 0)\n- timestamp: unix epoch timestamp in milliseconds for the message\n    (Kayrock client only, default: nil, must be using api_version >= 3)","ref":"KafkaEx.html#produce/4"},{"type":"function","title":"Example - KafkaEx.produce/4","doc":"```elixir\niex> KafkaEx.produce(\"bar\", 0, \"hey\")\n:ok\niex> KafkaEx.produce(\"foo\", 0, \"hey\", [worker_name: :pr, required_acks: 1])\n{:ok, 9771}\n```","ref":"KafkaEx.html#produce/4-example"},{"type":"function","title":"KafkaEx.start/2","doc":"","ref":"KafkaEx.html#start/2"},{"type":"function","title":"KafkaEx.start_link_worker/2","doc":"Start and link a worker outside of a supervision tree\n\nThis takes the same arguments as `create_worker/2` except that it adds\n\n- `server_impl` - This is the GenServer that will be used for the\n  client genserver implementation - e.g., `KafkaEx.Server0P8P0`,\n  `KafkaEx.Server0P10AndLater`, `KafkaEx.New.Client`.  Defaults to the value\n  determined by the `kafka_version` setting.","ref":"KafkaEx.html#start_link_worker/2"},{"type":"function","title":"KafkaEx.stop_worker/1","doc":"Stop a worker created with create_worker/2\n\nReturns `:ok` on success or `:error` if `worker` is not a valid worker","ref":"KafkaEx.html#stop_worker/1"},{"type":"function","title":"KafkaEx.stream/3","doc":"Returns a streamable struct that may be used for consuming messages.\n\nThe returned struct is compatible with the `Stream` and `Enum` modules.  Some\nimportant usage notes follow; see below for a detailed list of options.\n\n```elixir\niex> KafkaEx.produce(\"foo\", 0, \"hey\")\n:ok\niex> KafkaEx.produce(\"foo\", 0, \"hi\")\n:ok\niex> stream = KafkaEx.stream(\"foo\", 0)\n%KafkaEx.Stream{...}\niex> Enum.take(stream, 2)\n[%KafkaEx.Protocol.Fetch.Message{attributes: 0, crc: 1784030606, key: \"\",\n    offset: 0, value: \"hey\"},\n %KafkaEx.Protocol.Fetch.Message{attributes: 0, crc: 3776653906, key: \"\",\n     offset: 1, value: \"hi\"}]\niex> stream |> Stream.map(fn(msg) -> IO.puts(msg.value) end) |> Stream.run\n\"hey\"\n\"hi\"\n#  NOTE this will block!  See below.\n```","ref":"KafkaEx.html#stream/3"},{"type":"function","title":"Reusing streams - KafkaEx.stream/3","doc":"Reusing the same `KafkaEx.Stream` struct results in consuming the same\nmessages multiple times.  This is by design and mirrors the functionality of\n`File.stream!/3`.  If you want to reuse the same stream struct, update its\n`:offset` before reuse.\n\n```\niex> stream = KafkaEx.stream(\"foo\", 0)\niex> [m1, m2] = Enum.take(stream, 2)\niex> [m1, m2] = Enum.take(stream, 2)   # these will be the same messages\niex> stream = %{stream | fetch_request: %{stream.fetch_request | offset: m2.offset + 1}}\niex> [m3, m4] = Enum.take(stream, 2)   # new messages\n```","ref":"KafkaEx.html#stream/3-reusing-streams"},{"type":"function","title":"Streams block at log end - KafkaEx.stream/3","doc":"By default, the stream consumes indefinitely and will block at log end until\nnew messages are available.  Use the `no_wait_at_logend: true` option to have\nthe stream halt when no more messages are available.  This mirrors the\ncommand line arguments of\n[SimpleConsumerShell](https://cwiki.apache.org/confluence/display/KAFKA/System+Tools#SystemTools-SimpleConsumerShell).\n\nNote that this means that fetches will return up to as many messages\nas are immediately available in the partition, regardless of arguments.\n\n```\niex> Enum.map(1..3, fn(ix) -> KafkaEx.produce(\"bar\", 0, \"Msg #{ix}\") end)\niex> stream = KafkaEx.stream(\"bar\", 0, no_wait_at_logend: true, offset: 0)\niex> Enum.map(stream, fn(m) -> m.value end) # does not block\n[\"Msg 1\", \"Msg 2\", \"Msg 3\"]\niex> stream |> Stream.map(fn(m) -> m.value end) |> Enum.take(10)\n# only 3 messages are available\n[\"Msg 1\", \"Msg 2\", \"Msg 3\"]\n```","ref":"KafkaEx.html#stream/3-streams-block-at-log-end"},{"type":"function","title":"Consumer group and auto commit - KafkaEx.stream/3","doc":"If you pass a value for the `consumer_group` option and true for\n`auto_commit`, the offset of the last message consumed will be committed to\nthe broker during each cycle.\n\nFor example, suppose we start at the beginning of a partition with millions\nof messages and the `max_bytes` setting is such that each `fetch` request\ngets 25 messages.  In this setting, we will (roughly) be committing offsets\n25, 50, 75, etc.\n\nNote that offsets are committed immediately after messages are retrieved\nand before you know if you have successfully consumed them.  It is\ntherefore possible that you could miss messages if your consumer crashes in\nthe middle of consuming a batch, effectively losing the guarantee of\nat-least-once delivery.  If you need this guarantee, we recommend that you\nconstruct a GenServer-based consumer module and manage your commits manually.\n\n```\niex> Enum.map(1..10, fn(ix) -> KafkaEx.produce(\"baz\", 0, \"Msg #{ix}\") end)\niex> stream = KafkaEx.stream(\"baz\", 0, consumer_group: \"my_consumer\", auto_commit: true)\niex> stream |> Enum.take(2) |> Enum.map(fn(msg) -> msg.value end)\n[\"Msg 1\", \"Msg 2\"]\niex> stream |> Enum.take(2) |> Enum.map(fn(msg) -> msg.value end)\n[\"Msg 1\", \"Msg 2\"]  # same values\niex> stream2 = KafkaEx.stream(\"baz\", 0, consumer_group: \"my_consumer\", auto_commit: true)\niex> stream2 |> Enum.take(1) |> Enum.map(fn(msg) -> msg.value end)\n[\"Msg 3\"] # stream2 got the next available offset\n```","ref":"KafkaEx.html#stream/3-consumer-group-and-auto-commit"},{"type":"function","title":"Options - KafkaEx.stream/3","doc":"`KafkaEx.stream/3` accepts a keyword list of options for the third argument.\n\n- `no_wait_at_logend` (boolean): Set this to true to halt the stream when\nthere are no more messages available.  Defaults to false, i.e., the stream\nblocks to wait for new messages.\n\n- `worker_name` (term): The KafkaEx worker to use for communication with the\nbrokers.  Defaults to `:kafka_ex` (the default worker).\n\n- `consumer_group` (string): Name of the consumer group used for the initial\noffset fetch and automatic offset commit (if `auto_commit` is true). Omit\nthis value or use `:no_consumer_group` to not use a consumer group (default).\nConsumer groups are not compatible with Kafka < 0.8.2.\n\n- `offset` (integer): The offset from which to start fetching.  By default,\nthis is the last available offset of the partition when no consumer group is\nspecified.  When a consumer group is specified, the next message after the\nlast committed offset is used.  For Kafka < 0.8.2 you must explicitly specify\nan offset.\n\n- `auto_commit` (boolean): If true, the stream automatically commits offsets\nof fetched messages.  See discussion above.\n\n- `api_versions` (map): Allows overriding api versions for `:fetch`,\n`:offset_fetch`, and `:offset_commit` when using the Kayrock client.  Defaults to\n`%{fetch: 0, offset_fetch: 0, offset_commit: 0}`.  Use\n`%{fetch: 3, offset_fetch: 3, offset_commit: 3}` with the kayrock client to\nachieve offsets stored in Kafka (instead of zookeeper) and messages fetched\nwith timestamps.","ref":"KafkaEx.html#stream/3-options"},{"type":"function","title":"KafkaEx.sync_group/2","doc":"Sends a request to synchronize with a consumer group.","ref":"KafkaEx.html#sync_group/2"},{"type":"function","title":"KafkaEx.valid_consumer_group?/1","doc":"Returns true if the input is a valid consumer group or :no_consumer_group","ref":"KafkaEx.html#valid_consumer_group?/1"},{"type":"type","title":"KafkaEx.ssl_options/0","doc":"","ref":"KafkaEx.html#t:ssl_options/0"},{"type":"type","title":"KafkaEx.uri/0","doc":"","ref":"KafkaEx.html#t:uri/0"},{"type":"type","title":"KafkaEx.worker_init/0","doc":"","ref":"KafkaEx.html#t:worker_init/0"},{"type":"type","title":"KafkaEx.worker_setting/0","doc":"","ref":"KafkaEx.html#t:worker_setting/0"},{"type":"module","title":"KafkaEx.Compression","doc":"Handles compression/decompression of messages.\n\nSee https://cwiki.apache.org/confluence/display/KAFKA/A+Guide+To+The+Kafka+Protocol#AGuideToTheKafkaProtocol-Compression\n\nTo add new compression types:\n\n1. Add the appropriate dependency to mix.exs (don't forget to add it\nto the application list).\n2. Add the appropriate attribute value and compression_type atom.\n3. Add a decompress function clause.\n4. Add a compress function clause.","ref":"KafkaEx.Compression.html"},{"type":"function","title":"KafkaEx.Compression.compress/2","doc":"This function should pattern match on the compression_type atom and\nreturn the compressed data as well as the corresponding attribute\nvalue.","ref":"KafkaEx.Compression.html#compress/2"},{"type":"function","title":"KafkaEx.Compression.decompress/2","doc":"This function should pattern match on the attribute value and return\nthe decompressed data.","ref":"KafkaEx.Compression.html#decompress/2"},{"type":"function","title":"KafkaEx.Compression.snappy_decompress_chunk/2","doc":"","ref":"KafkaEx.Compression.html#snappy_decompress_chunk/2"},{"type":"type","title":"KafkaEx.Compression.attribute_t/0","doc":"","ref":"KafkaEx.Compression.html#t:attribute_t/0"},{"type":"type","title":"KafkaEx.Compression.compression_type_t/0","doc":"","ref":"KafkaEx.Compression.html#t:compression_type_t/0"},{"type":"module","title":"KafkaEx.Config","doc":"Configuring KafkaEx\n\n```\nimport Config\n\nconfig :kafka_ex,\n  # A list of brokers to connect to. This can be in either of the following formats\n  #\n  #  * [{\"HOST\", port}...]\n  #  * CSV - `\"HOST:PORT,HOST:PORT[,...]\"`\n  #  * {mod, fun, args}\n  #  * &arity_zero_fun/0\n  #  * fn -> ... end\n  #\n  # If you receive :leader_not_available\n  # errors when producing messages, it may be necessary to modify \"advertised.host.name\" in the\n  # server.properties file.\n  # In the case below you would set \"advertised.host.name=localhost\"\n  brokers: [\n    {\"localhost\", 9092},\n    {\"localhost\", 9093},\n    {\"localhost\", 9094}\n  ],\n  #\n  # OR:\n  # brokers: \"localhost:9092,localhost:9093,localhost:9094\"\n  #\n  # It may be useful to configure your brokers at runtime, for example if you use\n  # service discovery instead of storing your broker hostnames in a config file.\n  # To do this, you can use `{mod, fun, args}` or a zero-arity function, and `KafkaEx`\n  # will invoke your callback when fetching the `:brokers` configuration.\n  # Note that when using this approach you must return a list of host/port pairs.\n  #\n  # the default consumer group for worker processes, must be a binary (string)\n  #    NOTE if you are on Kafka < 0.8.2 or if you want to disable the use of\n  #    consumer groups, set this to :no_consumer_group (this is the\n  #    only exception to the requirement that this value be a binary)\n  consumer_group: \"kafka_ex\",\n  # The client_id is the logical grouping of a set of kafka clients.\n  client_id: \"kafka_ex\",\n  # Set this value to true if you do not want the default\n  # `KafkaEx.Server` worker to start during application start-up -\n  # i.e., if you want to start your own set of named workers\n  disable_default_worker: false,\n  # Timeout value, in msec, for synchronous operations (e.g., network calls).\n  # If this value is greater than GenServer's default timeout of 5000, it will also\n  # be used as the timeout for work dispatched via KafkaEx.Server.call (e.g., KafkaEx.metadata).\n  # In those cases, it should be considered a 'total timeout', encompassing both network calls and\n  # wait time for the genservers.\n  sync_timeout: 3000,\n  # Supervision max_restarts - the maximum amount of restarts allowed in a time frame\n  max_restarts: 10,\n  # Supervision max_seconds -  the time frame in which :max_restarts applies\n  max_seconds: 60,\n  # Interval in milliseconds that GenConsumer waits to commit offsets.\n  commit_interval: 5_000,\n  # Threshold number of messages consumed for GenConsumer to commit offsets\n  # to the broker.\n  commit_threshold: 100,\n  # The policy for resetting offsets when an :offset_out_of_range error occurs\n  # Options:\n  # - `:earliest` - Will move to the offset to the oldest available\n  # - `:latest` - Will move the offset to the most recent.\n  # - `:none` - The error will simply be raised\n  auto_offset_reset: :none,\n  # Interval in milliseconds to wait before reconnect to kafka\n  sleep_for_reconnect: 400,\n  # This is the flag that enables use of ssl\n  use_ssl: true,\n  # see SSL OPTION DESCRIPTIONS - CLIENT SIDE at http://erlang.org/doc/man/ssl.html\n  # for supported options\n  ssl_options: [\n    # Fix warnings. More at https://github.com/erlang/otp/issues/5352\n    verify: :verify_none,\n    cacertfile: File.cwd!() <> \"/ssl/ca-cert\",\n    certfile: File.cwd!() <> \"/ssl/cert.pem\",\n    keyfile: File.cwd!() <> \"/ssl/key.pem\"\n  ],\n  snappy_module: :snappyer,\n  # set this to the version of the kafka broker that you are using\n  # include only major.minor.patch versions.  must be at least 0.8.0\n  # use \"kayrock\" for the new client\n  kafka_version: \"0.10.1\"\n\nenv_config = Path.expand(\"#{Mix.env()}.exs\", __DIR__)\n\nif File.exists?(env_config) do\n  import_config(env_config)\nend\n```","ref":"KafkaEx.Config.html"},{"type":"module","title":"KafkaEx.ConsumerGroup","doc":"A process that manages membership in a Kafka consumer group.\n\nConsumers in a consumer group coordinate with each other through a Kafka\nbroker to distribute the work of consuming one or several topics without any\noverlap. This is facilitated by the\n[Kafka client-side assignment protocol](https://cwiki.apache.org/confluence/display/KAFKA/Kafka+Client-side+Assignment+Proposal).\n\nAny time group membership changes (a member joins or leaves the group), a\nKafka broker initiates group synchronization by asking one of the group\nmembers (the leader elected by the broker) to provide partition assignments\nfor the whole group.  KafkaEx uses a round robin partition assignment\nalgorithm by default.  This can be overridden by passing a callback function\nin the `:partition_assignment_callback` option.  See\n`KafkaEx.ConsumerGroup.PartitionAssignment` for details on partition\nassignment functions.\n\nA `KafkaEx.ConsumerGroup` process is responsible for:\n\n1. Maintaining membership in a Kafka consumer group.\n2. Determining partition assignments if elected as the group leader.\n3. Launching and terminating `KafkaEx.GenConsumer` processes based on its\n  assigned partitions.\n\nTo use a `KafkaEx.ConsumerGroup`, a developer must define a module that\nimplements the `KafkaEx.GenConsumer` behaviour and start a\n`KafkaEx.ConsumerGroup` configured to use that module.\n\nThe api versions of some of the underlying messages can be specified in the\n`:api_versions` option.  Note that these will be ignored (api version 0 used)\nunless you have `kafka_version: \"kayrock\"` set in the KafkaEx application\nconfig.  The following versions can be specified:\n\n* `:fetch` - Fetch requests - use v2+ for newer versions of Kafka\n* `:offset_fetch` - Offset fetch requests - use v1+ for offsets stored in\n  Kafka (as opposed to zookeeper)\n* `:offset_commit` - Offset commit requests - use v1+ to store offsets in\n  Kafka (as opposed to zookeeper)","ref":"KafkaEx.ConsumerGroup.html"},{"type":"module","title":"Example - KafkaEx.ConsumerGroup","doc":"Suppose we want to consume from a topic called `\"example_topic\"` with a\nconsumer group named `\"example_group\"` and we have a `KafkaEx.GenConsumer`\nimplementation called `ExampleGenConsumer` (see the `KafkaEx.GenConsumer`\ndocumentation).  We could start a consumer group in our application's\nsupervision tree as follows:\n\n```\ndefmodule MyApp do\n  use Application\n\n  def start(_type, _args) do\n    consumer_group_opts = [\n      # setting for the ConsumerGroup\n      heartbeat_interval: 1_000,\n      # this setting will be forwarded to the GenConsumer\n      commit_interval: 1_000\n    ]\n\n    gen_consumer_impl = ExampleGenConsumer\n    consumer_group_name = \"example_group\"\n    topic_names = [\"example_topic\"]\n\n    children = [\n      # ... other children\n      %{\n        id: KafkaEx.ConsumerGroup,\n        start: {\n          KafkaEx.ConsumerGroup,\n          :start_link,\n          [gen_consumer_impl, consumer_group_name, topic_names, consumer_group_opts]\n        }\n      }\n      # ... other children\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```\n\n**Note** It is not necessary for the Elixir nodes in a consumer group to be\nconnected (i.e., using distributed Erlang methods).  The coordination of\ngroup consumers is mediated by the broker.\n\nSee `start_link/4` for configuration details.","ref":"KafkaEx.ConsumerGroup.html#module-example"},{"type":"function","title":"KafkaEx.ConsumerGroup.active?/2","doc":"Returns true if at least one child consumer process is alive","ref":"KafkaEx.ConsumerGroup.html#active?/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.assignments/2","doc":"Returns a list of topic and partition assignments for which this consumer is\nresponsible.\n\nThese are assigned by the leader and communicated by the broker on sync.","ref":"KafkaEx.ConsumerGroup.html#assignments/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"KafkaEx.ConsumerGroup.html#child_spec/1"},{"type":"function","title":"KafkaEx.ConsumerGroup.consumer_pids/1","doc":"Returns the pids of consumer processes","ref":"KafkaEx.ConsumerGroup.html#consumer_pids/1"},{"type":"function","title":"KafkaEx.ConsumerGroup.consumer_supervisor_pid/2","doc":"Returns the pid of the `KafkaEx.GenConsumer.Supervisor` that supervises this\nmember's consumers.\n\nReturns `nil` if called before the initial sync.","ref":"KafkaEx.ConsumerGroup.html#consumer_supervisor_pid/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.generation_id/2","doc":"Returns the generation id of the consumer group.\n\nThe generation id is provided by the broker on sync.  Returns `nil` if\nqueried before the initial sync has completed.","ref":"KafkaEx.ConsumerGroup.html#generation_id/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.get_manager_pid/1","doc":"Returns the pid of the `KafkaEx.ConsumerGroup.Manager` process for the\ngiven consumer group supervisor.\n\nIntended for introspection usage only.","ref":"KafkaEx.ConsumerGroup.html#get_manager_pid/1"},{"type":"function","title":"KafkaEx.ConsumerGroup.group_name/2","doc":"Returns the name of the consumer group","ref":"KafkaEx.ConsumerGroup.html#group_name/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.leader?/2","doc":"Returns true if this consumer is the leader of the consumer group\n\nLeaders are elected by the broker and are responsible for assigning\npartitions.  Returns false if queried before the initial sync has completed.","ref":"KafkaEx.ConsumerGroup.html#leader?/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.leader_id/2","doc":"Returns the member id of the consumer group's leader\n\nThis is provided by the broker on sync.  Returns `nil` if queried before the\ninitial sync has completed","ref":"KafkaEx.ConsumerGroup.html#leader_id/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.member_id/2","doc":"Returns the consumer group member id\n\nThe id is assigned by the broker.  Returns `nil` if queried before the\ninitial sync has completed.","ref":"KafkaEx.ConsumerGroup.html#member_id/2"},{"type":"function","title":"KafkaEx.ConsumerGroup.partition_consumer_map/1","doc":"Returns a map from `{topic, partition_id}` to consumer pid","ref":"KafkaEx.ConsumerGroup.html#partition_consumer_map/1"},{"type":"function","title":"KafkaEx.ConsumerGroup.start_link/4","doc":"Starts a consumer group process tree process linked to the current process.\n\nThis can be used to start a `KafkaEx.ConsumerGroup` as part of a supervision\ntree.\n\n`consumer_module` is\n - a module that implements the `KafkaEx.GenConsumer`\nbehaviour.\n - a tuple of `{gen_consumer_module, consumer_module}` can substitute another\n   `GenServer` implementation for `KafkaEx.GenConsumer`. When a single module\n   is passed it is transformed to `{KafkaEx.GenConsumer, consumer_module}`.\n\n`group_name` is the name of the consumer group.\n\n`topics` is a list of topics that the consumer group should consume from.\n\n`opts` can be composed of options for the supervisor as well as for the\n`KafkEx.GenConsumer` processes that will be spawned by the supervisor.  See\n`t:option/0` for details.\n\n*Note* When starting a consumer group with multiple topics, you should\npropagate this configuration change to your consumers.  If you add a topic to\nan existing consumer group from a single consumer, it may take a long time\nto propagate depending on the leader election process.\n\n#","ref":"KafkaEx.ConsumerGroup.html#start_link/4"},{"type":"function","title":"Return Values - KafkaEx.ConsumerGroup.start_link/4","doc":"This function has the same return values as `Supervisor.start_link/3`.","ref":"KafkaEx.ConsumerGroup.html#start_link/4-return-values"},{"type":"type","title":"KafkaEx.ConsumerGroup.option/0","doc":"Option values used when starting a consumer group\n\n* `:heartbeat_interval` - How frequently, in milliseconds, to send heartbeats\n   to the broker.  This impacts how quickly we will process partition\n   changes as consumers start/stop.  Default: 5000 (5 seconds).\n* `:session_timeout` - Consumer group session timeout in milliseconds.\n   Default: 30000 (30 seconds).  See below.\n* `:session_timeout_padding` - Timeout padding for consumer group options.\n   Default: 10000 (10 seconds).  See below.\n* Any of `t:KafkaEx.GenConsumer.option/0`,\n   which will be passed on to consumers\n* `:gen_server_opts` - `t:GenServer.options/0` passed on to the manager\n   GenServer\n* `:name` - Name for the consumer group supervisor\n* `:max_restarts`, `:max_seconds` - Supervisor restart policy parameters\n* `:partition_assignment_callback` - See\n   `t:KafkaEx.ConsumerGroup.PartitionAssignment.callback/0`\n* `:uris` - See `KafkaEx.create_worker/2`\n\nNote `:session_timeout` is registered with the broker and determines how long\nbefore the broker will de-register a consumer from which it has not heard a\nheartbeat.  This value must between the broker cluster's configured values\nfor `group.min.session.timeout.ms` and `group.max.session.timeout.ms` (6000\nand 30000 by default).  See\n[https://kafka.apache.org/documentation/#configuration](https://kafka.apache.org/documentation/#configuration).\nYou may need to adjust `session_timeout_padding` on high-latency clusters to\navoid timing out when joining/syncing consumer groups.","ref":"KafkaEx.ConsumerGroup.html#t:option/0"},{"type":"type","title":"KafkaEx.ConsumerGroup.options/0","doc":"","ref":"KafkaEx.ConsumerGroup.html#t:options/0"},{"type":"module","title":"KafkaEx.ConsumerGroup.PartitionAssignment","doc":"Contains typespecs and reference algorithms for assigning partitions\n\n`round_robin/2` is used by `KafkaEx.ConsumerGroup` by default and should\nsuffice in most cases.\n\nFor custom assignments, any function matching the\n`t:callback/0` type spec can be used.","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html"},{"type":"function","title":"KafkaEx.ConsumerGroup.PartitionAssignment.round_robin/2","doc":"Round robin assignment\n\nIterates over the partitions and members, giving the first member the first\npartition, the second member the second partition, etc, looping back to the\nbeginning of the list of members when finished.\n\nExample:\n    iex> KafkaEx.ConsumerGroup.PartitionAssignment([\"m1\", \"m2\"], [{\"t1\", 0}, {\"t2, 1\"}, {\"t3\", 2}])\n    %{\"m1\" => [{\"t1\", 0}, {\"t3\", 2}], \"m2\" => [{\"t2\", 1}]}","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html#round_robin/2"},{"type":"type","title":"KafkaEx.ConsumerGroup.PartitionAssignment.callback/0","doc":"A function that can assign partitions.\n\n`members` is a list of member IDs and `partitions` is a list of partitions\nthat need to be assigned to a group member.\n\nThe return value must be a map with member IDs as keys and a list of\npartition assignments as values. For each member ID in the returned map, the\nassigned partitions will become the `assignments` argument to\n`KafkaEx.GenConsumer.Supervisor.start_link/4` in the corresponding member\nprocess. Any member that's omitted from the return value will not be assigned\nany partitions.\n\n#","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html#t:callback/0"},{"type":"type","title":"Example - KafkaEx.ConsumerGroup.PartitionAssignment.callback/0","doc":"Given the following `members` and `partitions` to be assigned:\n\n```\nmembers = [\"member1\", \"member2\", \"member3\"]\npartitions = [{\"topic\", 0}, {\"topic\", 1}, {\"topic\", 2}]\n```\n\nOne possible assignment is as follows:\n\n```\nExampleGenConsumer.assign_partitions(members, partitions)\n#=> %{\"member1\" => [{\"topic\", 0}, {\"topic\", 2}], \"member2\" => [{\"topic\", 1}]}\n```\n\nIn this case, the consumer group process for `\"member1\"` will launch two\n`KafkaEx.GenConsumer` processes (one for each of its assigned partitions),\n`\"member2\"` will launch one `KafkaEx.GenConsumer` process, and `\"member3\"` will\nlaunch no processes.","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html#t:callback/0-example"},{"type":"type","title":"KafkaEx.ConsumerGroup.PartitionAssignment.member_id/0","doc":"The ID (string) of a member of a consumer group, assigned by a Kafka broker.","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html#t:member_id/0"},{"type":"type","title":"KafkaEx.ConsumerGroup.PartitionAssignment.partition/0","doc":"A partition of a single topic (embeds the name of the topic).","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html#t:partition/0"},{"type":"type","title":"KafkaEx.ConsumerGroup.PartitionAssignment.partition_id/0","doc":"The integer ID of a partition of a Kafka topic.","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html#t:partition_id/0"},{"type":"type","title":"KafkaEx.ConsumerGroup.PartitionAssignment.topic/0","doc":"The string name of a Kafka topic.","ref":"KafkaEx.ConsumerGroup.PartitionAssignment.html#t:topic/0"},{"type":"module","title":"KafkaEx.DefaultPartitioner","doc":"Default partitioner implementation.\n\nWhen message key is set and partition isn't, partition is decided based\non murmur2 hash of a key to provide Java implementation consistency. When\nmessage key and partition is missing, partition is selected randomly.\nWhen partition is provided nothing changes.","ref":"KafkaEx.DefaultPartitioner.html"},{"type":"function","title":"KafkaEx.DefaultPartitioner.assign_partition/2","doc":"","ref":"KafkaEx.DefaultPartitioner.html#assign_partition/2"},{"type":"behaviour","title":"KafkaEx.GenConsumer","doc":"A behaviour module for implementing a Kafka consumer.\n\nA `KafkaEx.GenConsumer` is an Elixir process that consumes messages from\nKafka. A single `KafkaEx.GenConsumer` process consumes from a single\npartition of a Kafka topic. Several `KafkaEx.GenConsumer` processes can be\nused to consume from multiple partitions or even multiple topics. Partition\nassignments for a group of `KafkaEx.GenConsumer`s can be defined manually\nusing `KafkaEx.GenConsumer.Supervisor` or coordinated across a cluster of\nnodes using `KafkaEx.ConsumerGroup`.\n\nA `KafkaEx.GenConsumer` must implement three callbacks.  Two of these will be\ndefined with default behavior if you add `use KafkaEx.GenConsumer` to your\nmodule, leaving just `c:handle_message_set/2` to be implemented.  This is the\nrecommended usage.","ref":"KafkaEx.GenConsumer.html"},{"type":"behaviour","title":"Example - KafkaEx.GenConsumer","doc":"The `KafkaEx.GenConsumer` behaviour abstracts common Kafka consumer\ninteractions.  `KafkaEx.GenConsumer` will take care of the details of\ndetermining a starting offset, fetching messages from a Kafka broker, and\ncommitting offsets for consumed messages. Developers are only required to\nimplement `c:handle_message_set/2` to process messages.\n\nThe following is a minimal example that logs each message as it's consumed:\n\n```\ndefmodule ExampleGenConsumer do\n  use KafkaEx.GenConsumer\n\n  alias KafkaEx.Protocol.Fetch.Message\n\n  require Logger\n\n  # note - messages are delivered in batches\n  def handle_message_set(message_set, state) do\n    for %Message{value: message} <- message_set do\n      Logger.debug(fn -> \"message: \" <> inspect(message) end)\n    end\n    {:async_commit, state}\n  end\nend\n```\n\n`c:handle_message_set/2` will be called with the batch of messages fetched\nfrom the broker.  The number of messages in a batch is determined by the\nnumber of messages available and the `max_bytes` and `min_bytes` parameters\nof the fetch request (which can be configured in KafkaEx).  In this example,\nbecause `c:handle_message_set/2` always returns `{:async_commit, new_state}`,\nthe message offsets will be automatically committed asynchronously.","ref":"KafkaEx.GenConsumer.html#module-example"},{"type":"behaviour","title":"Committing Offsets - KafkaEx.GenConsumer","doc":"`KafkaEx.GenConsumer` manages a consumer's offsets by committing the offsets\nof consumed messages.  KafkaEx supports two commit strategies: asynchronous\nand synchronous.  The return value of `c:handle_message_set/2` determines\nwhich strategy is used:\n\n* `{:sync_commit, new_state}` causes synchronous offset commits.\n* `{:async_commit, new_state}` causes asynchronous offset commits.\n\nNote that with both of the offset commit strategies, only if the final offset\nin the message set is committed and this is done after the messages are\nconsumed.  If you want to commit the offset of every message consumed, use\nthe synchronous offset commit strategy and implement calls to\n`KafkaEx.offset_commit/2` within your consumer as appropriate.\n\n#","ref":"KafkaEx.GenConsumer.html#module-committing-offsets"},{"type":"behaviour","title":"Synchronous offset commits - KafkaEx.GenConsumer","doc":"When `c:handle_message_set/2` returns `{:sync_commit, new_state}`, the offset\nof the final message in the message set is committed immediately before\nfetching any more messages.  This strategy requires a significant amount of\ncommunication with the broker and could correspondingly degrade consumer\nperformance, but it will keep the offset commits tightly synchronized with\nthe consumer state.\n\nChoose the synchronous offset commit strategy if you want to favor\nconsistency of offset commits over performance, or if you have a low rate of\nmessage arrival.  The definition of a \"low rate\" depends on the situation,\nbut tens of messages per second could be considered a \"low rate\" in most\nsituations.\n\n#","ref":"KafkaEx.GenConsumer.html#module-synchronous-offset-commits"},{"type":"behaviour","title":"Asynchronous offset commits - KafkaEx.GenConsumer","doc":"When `c:handle_message_set/2` returns `{:async_commit, new_state}`, KafkaEx\nwill not commit offsets after every message set consumed.  To avoid\nexcessive network calls, the offsets are committed periodically (and when\nthe worker terminates).\n\nHow often a `KafkaEx.GenConsumer` auto-commits offsets is controlled by the two\nconfiguration values `:commit_interval` and `:commit_threshold`.\n\n* `:commit_interval` is the maximum time (in milliseconds) that a\n  `KafkaEx.GenConsumer` will delay committing the offset for an acknowledged\n  message.\n\n* `:commit_threshold` is the maximum number of acknowledged messages that a\n  `KafkaEx.GenConsumer` will allow to be uncommitted before triggering a\n  commit.\n\nThese can be set globally in the `:kafka_ex` app's environment or on a\nper-consumer basis by passing options to `start_link/5`:\n\n```\n# In config/config.exs\nconfig :kafka_ex,\n  commit_interval: 5000,\n  commit_threshold: 100\n\n# As options to start_link/5\nKafkaEx.GenConsumer.start_link(MyConsumer, \"my_group\", \"topic\", 0,\n                               commit_interval: 5000,\n                               commit_threshold: 100)\n```\n\nFor low-volume topics, `:commit_interval` is the dominant factor for how\noften a `KafkaEx.GenConsumer` auto-commits. For high-volume topics,\n`:commit_threshold` is the dominant factor.","ref":"KafkaEx.GenConsumer.html#module-asynchronous-offset-commits"},{"type":"behaviour","title":"Handler state and interaction - KafkaEx.GenConsumer","doc":"Use the `c:init/2` to initialize consumer state and `c:handle_call/3`,\n`c:handle_cast/2`, or `c:handle_info/2` to interact.\n\nExample:\n\n```\ndefmodule MyConsumer do\n  use KafkaEx.GenConsumer\n\n  defmodule State do\n    defstruct messages: [], calls: 0\n  end\n\n  def init(_topic, _partition) do\n    {:ok, %State{}}\n  end\n\n  def init(_topic, _partition, extra_args) do\n    {:ok, %State{}}\n  end\n\n  def handle_message_set(message_set, state) do\n    {:async_commit, %{state | messages: state.messages ++ message_set}}\n  end\n\n  def handle_call(:messages, _from, state) do\n    {:reply, state.messages, %{state | calls: state.calls + 1}}\n  end\nend\n\n{:ok, pid} = GenConsumer.start_link(MyConsumer, \"consumer_group\", \"topic\", 0)\nGenConsumer.call(pid, :messages)\n```\n\n**NOTE** If you do not implement `c:handle_call/3` or `c:handle_cast/2`, any\ncalls to `GenConsumer.call/3` or casts to `GenConsumer.cast/2` will raise an\nerror. Similarly, any messages sent to a `GenConsumer` will log an error if\nthere is no corresponding `c:handle_info/2` callback defined.","ref":"KafkaEx.GenConsumer.html#module-handler-state-and-interaction"},{"type":"behaviour","title":"Testing - KafkaEx.GenConsumer","doc":"A `KafkaEx.GenConsumer` can be unit-tested without a running Kafka broker by sending\nmessages directly to its `c:handle_message_set/2` function. The following\nrecipe can be used as a starting point when testing a `KafkaEx.GenConsumer`:\n\n```\ndefmodule ExampleGenConsumerTest do\n  use ExUnit.Case, async: true\n\n  alias KafkaEx.Protocol.Fetch.Message\n\n  @topic \"topic\"\n  @partition 0\n\n  setup do\n    {:ok, state} = ExampleGenConsumer.init(@topic, @partition)\n    {:ok, %{state: state}}\n  end\n\n  test \"it acks a message\", %{state: state} do\n    message_set = [%Message{offset: 0, value: \"hello\"}]\n    {response, _new_state} =\n      ExampleGenConsumer.handle_message_set(message_set, state)\n    assert response == :async_commit\n  end\nend\n```","ref":"KafkaEx.GenConsumer.html#module-testing"},{"type":"function","title":"KafkaEx.GenConsumer.call/3","doc":"Forwards a `GenServer.call/3` to the consumer implementation with the\nconsumer's state.\n\nThe implementation must return a `GenServer.call/3`-compatible value of the\nform `{:reply, reply_value, new_consumer_state}`.  The GenConsumer will\nturn this into an immediate timeout, which drives continued message\nconsumption.\n\nSee the moduledoc for an example.","ref":"KafkaEx.GenConsumer.html#call/3"},{"type":"function","title":"KafkaEx.GenConsumer.cast/2","doc":"Forwards a `GenServer.cast/2` to the consumer implementation with the\nconsumer's state.\n\nThe implementation must return a `GenServer.cast/2`-compatible value of the\nform `{:noreply, new_consumer_state}`. The GenConsumer will turn this into an\nimmediate timeout, which drives continued message consumption.","ref":"KafkaEx.GenConsumer.html#cast/2"},{"type":"function","title":"KafkaEx.GenConsumer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"KafkaEx.GenConsumer.html#child_spec/1"},{"type":"callback","title":"KafkaEx.GenConsumer.handle_call/3","doc":"Invoked by `KafkaEx.GenConsumer.call/3`.\n\nNote the default implementation will cause a `RuntimeError`.  If you want to\ninteract with your consumer, you must implement a handle_call function.","ref":"KafkaEx.GenConsumer.html#c:handle_call/3"},{"type":"callback","title":"KafkaEx.GenConsumer.handle_cast/2","doc":"Invoked by `KafkaEx.GenConsumer.cast/2`.\n\nNote the default implementation will cause a `RuntimeError`.  If you want to\ninteract with your consumer, you must implement a handle_cast function.","ref":"KafkaEx.GenConsumer.html#c:handle_cast/2"},{"type":"callback","title":"KafkaEx.GenConsumer.handle_info/2","doc":"Invoked by sending messages to the consumer.\n\nNote the default implementation will log error messages.  If you want to\ninteract with your consumer, you must implement a handle_info function.","ref":"KafkaEx.GenConsumer.html#c:handle_info/2"},{"type":"callback","title":"KafkaEx.GenConsumer.handle_message_set/2","doc":"Invoked for each message set consumed from a Kafka topic partition.\n\n`message_set` is a message set fetched from a Kafka broker and `state` is the\ncurrent state of the `KafkaEx.GenConsumer`.\n\nReturning `{:async_commit, new_state}` acknowledges `message` and continues\nto consume from the Kafka queue with new state `new_state`. Acknowledged\nmessages will be auto-committed (possibly at a later time) based on the\n`:commit_interval` and `:commit_threshold` options.\n\nReturning `{:sync_commit, new_state}` commits `message` synchronously before\ncontinuing to consume from the Kafka queue with new state `new_state`.\nCommitting a message synchronously means that no more messages will be\nconsumed until the message's offset is committed. `:sync_commit` should be\nused sparingly, since committing every message synchronously would impact a\nconsumer's performance and could result in excessive network traffic.","ref":"KafkaEx.GenConsumer.html#c:handle_message_set/2"},{"type":"function","title":"KafkaEx.GenConsumer.init/1","doc":"","ref":"KafkaEx.GenConsumer.html#init/1"},{"type":"callback","title":"KafkaEx.GenConsumer.init/2","doc":"Invoked when the server is started. `start_link/5` will block until it\nreturns.\n\n`topic` and `partition` are the arguments passed to `start_link/5`. They\nidentify the Kafka partition that the `KafkaEx.GenConsumer` will consume from.\n\nReturning `{:ok, state}` will cause `start_link/5` to return `{:ok, pid}` and\nthe process to start consuming from its assigned partition. `state` becomes\nthe consumer's state.\n\nAny other return value will cause the `start_link/5` to return `{:error,\nerror}` and the process to exit.","ref":"KafkaEx.GenConsumer.html#c:init/2"},{"type":"callback","title":"KafkaEx.GenConsumer.init/3","doc":"Invoked when the server is started. `start_link/5` will block until it\nreturns.\n\n`topic` and `partition` are the arguments passed to `start_link/5`. They\nidentify the Kafka partition that the `KafkaEx.GenConsumer` will consume from.\n\n`extra_args` is the value of the `extra_consumer_args` option to `start_link/5`.\n\nThe default implementation of this function calls `init/2`.\n\nReturning `{:ok, state}` will cause `start_link/5` to return `{:ok, pid}` and\nthe process to start consuming from its assigned partition. `state` becomes\nthe consumer's state.\n\nAny other return value will cause the `start_link/5` to return `{:error,\nerror}` and the process to exit.","ref":"KafkaEx.GenConsumer.html#c:init/3"},{"type":"function","title":"KafkaEx.GenConsumer.partition/2","doc":"Returns the topic and partition id for this consumer process","ref":"KafkaEx.GenConsumer.html#partition/2"},{"type":"function","title":"KafkaEx.GenConsumer.start_link/5","doc":"Starts a `KafkaEx.GenConsumer` process linked to the current process.\n\nThis can be used to start the `KafkaEx.GenConsumer` as part of a supervision tree.\n\nOnce the consumer has been started, the `c:init/2` function of\n`consumer_module` is called with `topic` and `partition` as its arguments.\n`group_name` is the consumer group name that will be used for managing\nconsumer offsets.\n\n#","ref":"KafkaEx.GenConsumer.html#start_link/5"},{"type":"function","title":"Options - KafkaEx.GenConsumer.start_link/5","doc":"* `:commit_interval` - The interval in milliseconds that the consumer will\n  wait to commit offsets of handled messages.  Default 5_000.\n\n* `:commit_threshold` - Threshold number of messages consumed to commit\n  offsets to the broker.  Default 100.\n\n* `:auto_offset_reset` - The policy for resetting offsets when an\n  `:offset_out_of_range` error occurs. `:earliest` will move the offset to\n  the oldest available, `:latest` moves to the most recent. If anything else\n  is specified, the error will simply be raised.\n\n* `:fetch_options` - Optional keyword list that is passed along to the\n  `KafkaEx.fetch` call.\n\n* `:extra_consumer_args` - Optional parameter that is passed along to the\n  `GenConsumer.init` call in the consumer module. Note that if `init/3` is not\n  implemented, the default implementation calls to `init/2`, dropping the extra\n  arguments.\n\n**NOTE** `:commit_interval`, `auto_commit_reset` and `:commit_threshold` default to the\napplication config (e.g., `Application.get_env/2`) if that value is present, or the stated\ndefault if the application config is not present.\n\nAny valid options for `GenServer.start_link/3` can also be specified.\n\n#","ref":"KafkaEx.GenConsumer.html#start_link/5-options"},{"type":"function","title":"Return Values - KafkaEx.GenConsumer.start_link/5","doc":"This function has the same return values as `GenServer.start_link/3`.","ref":"KafkaEx.GenConsumer.html#start_link/5-return-values"},{"type":"type","title":"KafkaEx.GenConsumer.option/0","doc":"Option values used when starting a `KafkaEx.GenConsumer`.","ref":"KafkaEx.GenConsumer.html#t:option/0"},{"type":"type","title":"KafkaEx.GenConsumer.options/0","doc":"Options used when starting a `KafkaEx.GenConsumer`.","ref":"KafkaEx.GenConsumer.html#t:options/0"},{"type":"module","title":"KafkaEx.GenConsumer.Supervisor","doc":"A supervisor for managing `GenConsumer` processes that are part of a consumer\ngroup.\n\nThe supervisor will launch individual `GenConsumer` processes for each\npartition given by the `partitions` argument to `start_link/4`. When\nterminated, each of the supervisor's child processes will commit its latest\noffset before terminating.\n\nThis module manages a static list of consumer processes. For dynamically\ndistributing consumers in a consumer group across a cluster of nodes, see\n`KafkaEx.ConsumerGroup`.","ref":"KafkaEx.GenConsumer.Supervisor.html"},{"type":"function","title":"KafkaEx.GenConsumer.Supervisor.active?/1","doc":"Returns true if any child pids are alive","ref":"KafkaEx.GenConsumer.Supervisor.html#active?/1"},{"type":"function","title":"KafkaEx.GenConsumer.Supervisor.child_pids/1","doc":"Returns a list of child pids\n\nIntended to be used for operational and testing purposes","ref":"KafkaEx.GenConsumer.Supervisor.html#child_pids/1"},{"type":"function","title":"KafkaEx.GenConsumer.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"KafkaEx.GenConsumer.Supervisor.html#child_spec/1"},{"type":"function","title":"KafkaEx.GenConsumer.Supervisor.start_link/1","doc":"Starts a `GenConsumer.Supervisor` process linked to the current process.\n\n`gen_consumer_module` is a module that implements the `GenServer` behaviour\nwhich consumes events from Kafka.\n\n`consumer_module` is a module that implements the `GenConsumer` behaviour.\n\n`group_name` is the name of a consumer group.\n\n`assignments` is a list of partitions for the `GenConsumer`s to consume.\n\n`opts` accepts the same options as `KafkaEx.GenConsumer.start_link/5`.\n\n#","ref":"KafkaEx.GenConsumer.Supervisor.html#start_link/1"},{"type":"function","title":"Return Values - KafkaEx.GenConsumer.Supervisor.start_link/1","doc":"This function has the same return values as `Supervisor.start_link/3`.\n\nIf the supervisor and its consumers are successfully created, this function\nreturns `{:ok, pid}`, where `pid` is the PID of the supervisor.","ref":"KafkaEx.GenConsumer.Supervisor.html#start_link/1-return-values"},{"type":"function","title":"KafkaEx.GenConsumer.Supervisor.start_link/4","doc":"Starts a `GenConsumer.Supervisor` process linked to the current process.\n\nRefer to `start_link/1` for documentation of each parameter.\n\n#","ref":"KafkaEx.GenConsumer.Supervisor.html#start_link/4"},{"type":"function","title":"Return Values - KafkaEx.GenConsumer.Supervisor.start_link/4","doc":"Same as `start_link/1`.","ref":"KafkaEx.GenConsumer.Supervisor.html#start_link/4-return-values"},{"type":"module","title":"KafkaEx.LegacyPartitioner","doc":"Legacy default partitioner implementation.\n\nThis \"legacy partitioner\" used to be the default partitioner for KafkaEx\nwhen a message key is set and a partition isn't. This was intended to match\nthe behaviour of the default Java client, however there was some small\ndifferences. This legacy partitioner keeps the previous behaviour, and the\ncurrent default partitioner now correctly matches the Java client.","ref":"KafkaEx.LegacyPartitioner.html"},{"type":"function","title":"KafkaEx.LegacyPartitioner.assign_partition/2","doc":"","ref":"KafkaEx.LegacyPartitioner.html#assign_partition/2"},{"type":"module","title":"KafkaEx.NetworkClient","doc":"KafkaEx implementation of Client used to connect to Kafka Broker","ref":"KafkaEx.NetworkClient.html"},{"type":"behaviour","title":"KafkaEx.NetworkClient.Behaviour","doc":"Behaviour for any network client.\nCreated mainly to allow mocking request & responses in unit tests.","ref":"KafkaEx.NetworkClient.Behaviour.html"},{"type":"callback","title":"KafkaEx.NetworkClient.Behaviour.close_socket/1","doc":"Close socket, if socket is nil, do nothing.","ref":"KafkaEx.NetworkClient.Behaviour.html#c:close_socket/1"},{"type":"callback","title":"KafkaEx.NetworkClient.Behaviour.create_socket/2","doc":"Creates a socket to the given host and port.","ref":"KafkaEx.NetworkClient.Behaviour.html#c:create_socket/2"},{"type":"callback","title":"KafkaEx.NetworkClient.Behaviour.create_socket/3","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#c:create_socket/3"},{"type":"callback","title":"KafkaEx.NetworkClient.Behaviour.create_socket/4","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#c:create_socket/4"},{"type":"callback","title":"KafkaEx.NetworkClient.Behaviour.format_host/1","doc":"Returns the host in Erlang IP address format if the host is an IP address.\nOtherwise, returns the host as a char list.","ref":"KafkaEx.NetworkClient.Behaviour.html#c:format_host/1"},{"type":"callback","title":"KafkaEx.NetworkClient.Behaviour.send_async_request/2","doc":"Send request asynchronously to broker.","ref":"KafkaEx.NetworkClient.Behaviour.html#c:send_async_request/2"},{"type":"callback","title":"KafkaEx.NetworkClient.Behaviour.send_sync_request/3","doc":"Send request synchronously to broker.","ref":"KafkaEx.NetworkClient.Behaviour.html#c:send_sync_request/3"},{"type":"type","title":"KafkaEx.NetworkClient.Behaviour.host/0","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#t:host/0"},{"type":"type","title":"KafkaEx.NetworkClient.Behaviour.host_port/0","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#t:host_port/0"},{"type":"type","title":"KafkaEx.NetworkClient.Behaviour.kafka_ex_broker/0","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#t:kafka_ex_broker/0"},{"type":"type","title":"KafkaEx.NetworkClient.Behaviour.kafka_ex_socket/0","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#t:kafka_ex_socket/0"},{"type":"type","title":"KafkaEx.NetworkClient.Behaviour.request_data/0","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#t:request_data/0"},{"type":"type","title":"KafkaEx.NetworkClient.Behaviour.response_data/0","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#t:response_data/0"},{"type":"type","title":"KafkaEx.NetworkClient.Behaviour.use_ssl/0","doc":"","ref":"KafkaEx.NetworkClient.Behaviour.html#t:use_ssl/0"},{"type":"module","title":"KafkaEx.New.Client","doc":"Kayrock-compatible KafkaEx.Server implementation\n\nThis implementation attempts to keep as much Kafka 'business logic' as possible\nout of the server implementation, with the motivation that this should make\nthe client easier to maintain as the Kafka protocol evolves.\n\nThis implementation does, however, include implementations of all of the\nlegacy KafkaEx.Server `handle_call` clauses so that it can be compatible with\nthe legacy KafkaEx API.","ref":"KafkaEx.New.Client.html"},{"type":"function","title":"KafkaEx.New.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"KafkaEx.New.Client.html#child_spec/1"},{"type":"function","title":"KafkaEx.New.Client.send_request/4","doc":"Send a Kayrock request to the appropriate broker\n\nBroker metadata will be updated if necessary","ref":"KafkaEx.New.Client.html#send_request/4"},{"type":"function","title":"KafkaEx.New.Client.start_link/2","doc":"Start the server in a supervision tree","ref":"KafkaEx.New.Client.html#start_link/2"},{"type":"type","title":"KafkaEx.New.Client.args/0","doc":"","ref":"KafkaEx.New.Client.html#t:args/0"},{"type":"behaviour","title":"KafkaEx.New.Client.Protocol","doc":"This module is responsible for defining the behaviour of a protocol.","ref":"KafkaEx.New.Client.Protocol.html"},{"type":"callback","title":"KafkaEx.New.Client.Protocol.build_request/3","doc":"","ref":"KafkaEx.New.Client.Protocol.html#c:build_request/3"},{"type":"callback","title":"KafkaEx.New.Client.Protocol.parse_response/2","doc":"","ref":"KafkaEx.New.Client.Protocol.html#c:parse_response/2"},{"type":"type","title":"KafkaEx.New.Client.Protocol.api_version/0","doc":"","ref":"KafkaEx.New.Client.Protocol.html#t:api_version/0"},{"type":"type","title":"KafkaEx.New.Client.Protocol.consumer_group/0","doc":"","ref":"KafkaEx.New.Client.Protocol.html#t:consumer_group/0"},{"type":"type","title":"KafkaEx.New.Client.Protocol.params/0","doc":"","ref":"KafkaEx.New.Client.Protocol.html#t:params/0"},{"type":"module","title":"KafkaEx.New.Client.RequestBuilder","doc":"This module is used to build request for KafkaEx.New.Client.\nIt's main decision point which protocol to use for building request and what\nis required version.","ref":"KafkaEx.New.Client.RequestBuilder.html"},{"type":"function","title":"KafkaEx.New.Client.RequestBuilder.describe_groups_request/2","doc":"Builds request for Describe Groups API","ref":"KafkaEx.New.Client.RequestBuilder.html#describe_groups_request/2"},{"type":"module","title":"KafkaEx.New.Client.ResponseParser","doc":"This module is used to parse response from KafkaEx.New.Client.\nIt's main decision point which protocol to use for parsing response","ref":"KafkaEx.New.Client.ResponseParser.html"},{"type":"function","title":"KafkaEx.New.Client.ResponseParser.describe_groups_response/1","doc":"Parses response for Describe Groups API","ref":"KafkaEx.New.Client.ResponseParser.html#describe_groups_response/1"},{"type":"module","title":"KafkaEx.New.KafkaExAPI","doc":"This module interfaces Kafka through the New.Client implementation\n\nThis is intended to become the future KafkaEx API\n\nMost functions here take a client pid as the first argument.\n\n```\n{:ok, client} = KafkaEx.New.Client.start_link()\n\nKafkaEx.New.KafkaExAPI.latest_offset(client, \"some_topic\", 0)\n```","ref":"KafkaEx.New.KafkaExAPI.html"},{"type":"function","title":"KafkaEx.New.KafkaExAPI.cluster_metadata/1","doc":"Returns the cluster metadata from the given client","ref":"KafkaEx.New.KafkaExAPI.html#cluster_metadata/1"},{"type":"function","title":"KafkaEx.New.KafkaExAPI.correlation_id/1","doc":"Returns the current correlation id for the given client","ref":"KafkaEx.New.KafkaExAPI.html#correlation_id/1"},{"type":"function","title":"KafkaEx.New.KafkaExAPI.describe_group/2","doc":"Sends a request to describe a group identified by its name.\nWe support only one consumer group per request for now, as we don't\ngroup requests by group coordinator.","ref":"KafkaEx.New.KafkaExAPI.html#describe_group/2"},{"type":"function","title":"KafkaEx.New.KafkaExAPI.latest_offset/3","doc":"Fetch the latest offset for a given partition","ref":"KafkaEx.New.KafkaExAPI.html#latest_offset/3"},{"type":"function","title":"KafkaEx.New.KafkaExAPI.set_consumer_group_for_auto_commit/2","doc":"Set the consumer group name that will be used by the given client for\nautocommit\n\nNOTE this function will not be supported after the legacy API is removed","ref":"KafkaEx.New.KafkaExAPI.html#set_consumer_group_for_auto_commit/2"},{"type":"function","title":"KafkaEx.New.KafkaExAPI.topics_metadata/3","doc":"Get topic metadata for the given topics\n\nAlways calls out to the broker to get the most up-to-date metadata (and\nsubsequently updates the client's state with the updated metadata). Set\nallow_topic_creation to true to allow the topics to be created if they\ndon't exist","ref":"KafkaEx.New.KafkaExAPI.html#topics_metadata/3"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.client/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:client/0"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.consumer_group_name/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:consumer_group_name/0"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.correlation_id/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:correlation_id/0"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.error_atom/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:error_atom/0"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.node_id/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:node_id/0"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.offset/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:offset/0"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.partition_id/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:partition_id/0"},{"type":"type","title":"KafkaEx.New.KafkaExAPI.topic_name/0","doc":"","ref":"KafkaEx.New.KafkaExAPI.html#t:topic_name/0"},{"type":"protocol","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups","doc":"This module handles Describe Groups request & response parsing.\nRequest is built using Kayrock protocol, response is parsed to\nnative KafkaEx structs.","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.html"},{"type":"type","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.t/0","doc":"All the types that implement this protocol.","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.html#t:t/0"},{"type":"protocol","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Request","doc":"This protocol is used to build Describe Groups request","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Request.html"},{"type":"function","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Request.build_request/2","doc":"","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Request.html#build_request/2"},{"type":"type","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Request.t/0","doc":"All the types that implement this protocol.","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Request.html#t:t/0"},{"type":"protocol","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Response","doc":"This protocol is used to parse Describe Groups response","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Response.html"},{"type":"function","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Response.parse_response/1","doc":"","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Response.html#parse_response/1"},{"type":"type","title":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Response.t/0","doc":"All the types that implement this protocol.","ref":"KafkaEx.New.Protocols.Kayrock.DescribeGroups.Response.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Protocols.KayrockProtocol","doc":"This module handles Kayrock request & response handling & parsing.\nOnce Kafka Ex v1.0 is released, this module will be renamed to KayrockProtocol\nand will become a separated package.","ref":"KafkaEx.New.Protocols.KayrockProtocol.html"},{"type":"function","title":"KafkaEx.New.Protocols.KayrockProtocol.build_request/3","doc":"Builds request for Describe Groups API","ref":"KafkaEx.New.Protocols.KayrockProtocol.html#build_request/3"},{"type":"function","title":"KafkaEx.New.Protocols.KayrockProtocol.parse_response/2","doc":"Parses response for Describe Groups API","ref":"KafkaEx.New.Protocols.KayrockProtocol.html#parse_response/2"},{"type":"module","title":"KafkaEx.New.Structs.Broker","doc":"Encapsulates what we know about a broker and our connection","ref":"KafkaEx.New.Structs.Broker.html"},{"type":"function","title":"KafkaEx.New.Structs.Broker.has_socket?/2","doc":"","ref":"KafkaEx.New.Structs.Broker.html#has_socket?/2"},{"type":"type","title":"KafkaEx.New.Structs.Broker.t/0","doc":"","ref":"KafkaEx.New.Structs.Broker.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Structs.ClusterMetadata","doc":"Encapsulates what we know about the state of a Kafka broker cluster\n\nThis module is mainly used internally in New.Client, but some of its\nfunctions may be useful for extracting metadata information","ref":"KafkaEx.New.Structs.ClusterMetadata.html"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.broker_by_node_id/2","doc":"Returns a `t:Broker.t/0` for the given `t:KafkaExAPI.node_id/0` or `nil` if\nthere is no known broker with that node id","ref":"KafkaEx.New.Structs.ClusterMetadata.html#broker_by_node_id/2"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.brokers/1","doc":"Return a list of the known brokers","ref":"KafkaEx.New.Structs.ClusterMetadata.html#brokers/1"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.from_metadata_v1_response/1","doc":"Constructs a `t:t/0` from a `Kayrock.Metadata.V1.Response` struct.\n\nThe `V1` here is a minimum - this should work with higher versions of the\nmetadata response struct.","ref":"KafkaEx.New.Structs.ClusterMetadata.html#from_metadata_v1_response/1"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.known_topics/1","doc":"List names of topics known by the cluster metadata\n\nNOTE this is a subset of the topics in the cluster - it will only contain\ntopics for which we have fetched metadata","ref":"KafkaEx.New.Structs.ClusterMetadata.html#known_topics/1"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.put_consumer_group_coordinator/3","doc":"update a consumer group coordinator node id","ref":"KafkaEx.New.Structs.ClusterMetadata.html#put_consumer_group_coordinator/3"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.remove_topics/2","doc":"remove the given topics (e.g., when they are deleted)","ref":"KafkaEx.New.Structs.ClusterMetadata.html#remove_topics/2"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.select_node/2","doc":"Find the node id for a given selector\n\nNote this will not update the metadata, only select a node given the current metadata.\n\nSee `t:KafkaEx.New.Structs.NodeSelector.t/0`","ref":"KafkaEx.New.Structs.ClusterMetadata.html#select_node/2"},{"type":"function","title":"KafkaEx.New.Structs.ClusterMetadata.topics_metadata/2","doc":"Return the metadata for the given topics","ref":"KafkaEx.New.Structs.ClusterMetadata.html#topics_metadata/2"},{"type":"type","title":"KafkaEx.New.Structs.ClusterMetadata.node_select_error/0","doc":"Possible errors given by `select_node/2`","ref":"KafkaEx.New.Structs.ClusterMetadata.html#t:node_select_error/0"},{"type":"type","title":"KafkaEx.New.Structs.ClusterMetadata.t/0","doc":"","ref":"KafkaEx.New.Structs.ClusterMetadata.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Structs.ConsumerGroup","doc":"Encapsulates what we know about consumer group","ref":"KafkaEx.New.Structs.ConsumerGroup.html"},{"type":"function","title":"KafkaEx.New.Structs.ConsumerGroup.from_describe_group_response/1","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.html#from_describe_group_response/1"},{"type":"type","title":"KafkaEx.New.Structs.ConsumerGroup.t/0","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Structs.ConsumerGroup.Member","doc":"Encapsulates what we know about a consumer group member","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.html"},{"type":"function","title":"KafkaEx.New.Structs.ConsumerGroup.Member.from_describe_group_response/1","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.html#from_describe_group_response/1"},{"type":"type","title":"KafkaEx.New.Structs.ConsumerGroup.Member.partial_response/0","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.html#t:partial_response/0"},{"type":"type","title":"KafkaEx.New.Structs.ConsumerGroup.Member.t/0","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment","doc":"Encapsulates what we know about a consumer group.\nThe current assignment provided by the group leader (will only be present if the group is stable).","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.html"},{"type":"function","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.from_describe_group_response/1","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.html#from_describe_group_response/1"},{"type":"type","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.response_partial/0","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.html#t:response_partial/0"},{"type":"type","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.t/0","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment","doc":"Encapsulates what we know about a consumer group member partition assignment.\nWill only be present if the group is stable and is assigned to given topic.","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment.html"},{"type":"function","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment.from_describe_group_response/1","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment.html#from_describe_group_response/1"},{"type":"type","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment.response_partial/0","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment.html#t:response_partial/0"},{"type":"type","title":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment.t/0","doc":"","ref":"KafkaEx.New.Structs.ConsumerGroup.Member.MemberAssignment.PartitionAssignment.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Structs.NodeSelector","doc":"Defines node selector functions and macros","ref":"KafkaEx.New.Structs.NodeSelector.html"},{"type":"function","title":"KafkaEx.New.Structs.NodeSelector.consumer_group/1","doc":"Select the controller for the given consumer group","ref":"KafkaEx.New.Structs.NodeSelector.html#consumer_group/1"},{"type":"function","title":"KafkaEx.New.Structs.NodeSelector.controller/0","doc":"Select the cluster's controller node","ref":"KafkaEx.New.Structs.NodeSelector.html#controller/0"},{"type":"function","title":"KafkaEx.New.Structs.NodeSelector.first_available/0","doc":"Select first available node","ref":"KafkaEx.New.Structs.NodeSelector.html#first_available/0"},{"type":"function","title":"KafkaEx.New.Structs.NodeSelector.node_id/1","doc":"Select a specific node","ref":"KafkaEx.New.Structs.NodeSelector.html#node_id/1"},{"type":"function","title":"KafkaEx.New.Structs.NodeSelector.random/0","doc":"Select a random node","ref":"KafkaEx.New.Structs.NodeSelector.html#random/0"},{"type":"function","title":"KafkaEx.New.Structs.NodeSelector.topic_partition/2","doc":"Select the controller for the given topic and partition","ref":"KafkaEx.New.Structs.NodeSelector.html#topic_partition/2"},{"type":"type","title":"KafkaEx.New.Structs.NodeSelector.t/0","doc":"","ref":"KafkaEx.New.Structs.NodeSelector.html#t:t/0"},{"type":"type","title":"KafkaEx.New.Structs.NodeSelector.valid_strategy/0","doc":"","ref":"KafkaEx.New.Structs.NodeSelector.html#t:valid_strategy/0"},{"type":"module","title":"KafkaEx.New.Structs.Partition","doc":"Encapsulates what we know about a partition","ref":"KafkaEx.New.Structs.Partition.html"},{"type":"type","title":"KafkaEx.New.Structs.Partition.t/0","doc":"","ref":"KafkaEx.New.Structs.Partition.html#t:t/0"},{"type":"module","title":"KafkaEx.New.Structs.Topic","doc":"Encapsulates what we know about a topic","ref":"KafkaEx.New.Structs.Topic.html"},{"type":"type","title":"KafkaEx.New.Structs.Topic.t/0","doc":"","ref":"KafkaEx.New.Structs.Topic.html#t:t/0"},{"type":"behaviour","title":"KafkaEx.Partitioner","doc":"Behaviour definition for partitioners, that assigns partitions for requests.","ref":"KafkaEx.Partitioner.html"},{"type":"callback","title":"KafkaEx.Partitioner.assign_partition/2","doc":"","ref":"KafkaEx.Partitioner.html#c:assign_partition/2"},{"type":"function","title":"KafkaEx.Partitioner.get_key/1","doc":"Returns key for given messages\n\nFunction looks for message key in messages list of {ProduceRequest}. It may return\neither `{:ok, nil}` if no key was found, `{:ok, key}` when key was found,\nor `{:error, atom}` when error happens while looking for the key.","ref":"KafkaEx.Partitioner.html#get_key/1"},{"type":"module","title":"KafkaEx.Protocol.ApiVersions","doc":"Implementation of the Kafka ApiVersions request and response APIs\n\nSee: https://kafka.apache.org/protocol.html#The_Messages_ApiVersions\n\nReference implementation in Java:\nhttps://github.com/apache/kafka/blob/trunk/clients/src/main/java/org/apache/kafka/common/requests/ApiVersionsRequest.java\nhttps://github.com/apache/kafka/blob/trunk/clients/src/main/java/org/apache/kafka/common/requests/ApiVersionsResponse.java","ref":"KafkaEx.Protocol.ApiVersions.html"},{"type":"function","title":"KafkaEx.Protocol.ApiVersions.create_request/3","doc":"","ref":"KafkaEx.Protocol.ApiVersions.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.ApiVersions.parse_response/2","doc":"","ref":"KafkaEx.Protocol.ApiVersions.html#parse_response/2"},{"type":"module","title":"KafkaEx.Protocol.Common","doc":"A collection of common request generation and response parsing functions for the\nKafka wire protocol.","ref":"KafkaEx.Protocol.Common.html"},{"type":"function","title":"KafkaEx.Protocol.Common.encode_nullable_string/1","doc":"","ref":"KafkaEx.Protocol.Common.html#encode_nullable_string/1"},{"type":"function","title":"KafkaEx.Protocol.Common.encode_string/1","doc":"","ref":"KafkaEx.Protocol.Common.html#encode_string/1"},{"type":"function","title":"KafkaEx.Protocol.Common.map_encode/2","doc":"","ref":"KafkaEx.Protocol.Common.html#map_encode/2"},{"type":"function","title":"KafkaEx.Protocol.Common.parse_topics/3","doc":"","ref":"KafkaEx.Protocol.Common.html#parse_topics/3"},{"type":"function","title":"KafkaEx.Protocol.Common.read_array/3","doc":"","ref":"KafkaEx.Protocol.Common.html#read_array/3"},{"type":"function","title":"KafkaEx.Protocol.Common.topic_data/1","doc":"Generate the wire representation for a list of topics.","ref":"KafkaEx.Protocol.Common.html#topic_data/1"},{"type":"module","title":"KafkaEx.Protocol.ConsumerMetadata","doc":"Implementation of the Kafka ConsumerMetadata request and response APIs","ref":"KafkaEx.Protocol.ConsumerMetadata.html"},{"type":"function","title":"KafkaEx.Protocol.ConsumerMetadata.create_request/3","doc":"","ref":"KafkaEx.Protocol.ConsumerMetadata.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.ConsumerMetadata.parse_response/1","doc":"","ref":"KafkaEx.Protocol.ConsumerMetadata.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.CreateTopics","doc":"Implementation of the Kafka CreateTopics request and response APIs\n\nSee: https://kafka.apache.org/protocol.html#The_Messages_CreateTopics","ref":"KafkaEx.Protocol.CreateTopics.html"},{"type":"function","title":"KafkaEx.Protocol.CreateTopics.api_version/1","doc":"","ref":"KafkaEx.Protocol.CreateTopics.html#api_version/1"},{"type":"function","title":"KafkaEx.Protocol.CreateTopics.create_request/4","doc":"","ref":"KafkaEx.Protocol.CreateTopics.html#create_request/4"},{"type":"function","title":"KafkaEx.Protocol.CreateTopics.parse_response/2","doc":"","ref":"KafkaEx.Protocol.CreateTopics.html#parse_response/2"},{"type":"module","title":"KafkaEx.Protocol.DeleteTopics","doc":"Implementation of the Kafka DeleteTopics request and response APIs\n\nSee: https://kafka.apache.org/protocol.html#The_Messages_DeleteTopics","ref":"KafkaEx.Protocol.DeleteTopics.html"},{"type":"function","title":"KafkaEx.Protocol.DeleteTopics.api_version/1","doc":"","ref":"KafkaEx.Protocol.DeleteTopics.html#api_version/1"},{"type":"function","title":"KafkaEx.Protocol.DeleteTopics.create_request/4","doc":"","ref":"KafkaEx.Protocol.DeleteTopics.html#create_request/4"},{"type":"function","title":"KafkaEx.Protocol.DeleteTopics.parse_response/2","doc":"","ref":"KafkaEx.Protocol.DeleteTopics.html#parse_response/2"},{"type":"module","title":"KafkaEx.Protocol.Fetch","doc":"Implementation of the Kafka Fetch request and response APIs","ref":"KafkaEx.Protocol.Fetch.html"},{"type":"function","title":"KafkaEx.Protocol.Fetch.create_request/1","doc":"","ref":"KafkaEx.Protocol.Fetch.html#create_request/1"},{"type":"function","title":"KafkaEx.Protocol.Fetch.parse_partitions/4","doc":"","ref":"KafkaEx.Protocol.Fetch.html#parse_partitions/4"},{"type":"function","title":"KafkaEx.Protocol.Fetch.parse_response/1","doc":"","ref":"KafkaEx.Protocol.Fetch.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.Heartbeat","doc":"Implementation of the Kafka Heartbeat request and response APIs","ref":"KafkaEx.Protocol.Heartbeat.html"},{"type":"function","title":"KafkaEx.Protocol.Heartbeat.create_request/3","doc":"","ref":"KafkaEx.Protocol.Heartbeat.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.Heartbeat.parse_response/1","doc":"","ref":"KafkaEx.Protocol.Heartbeat.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.JoinGroup","doc":"Implementation of the Kafka JoinGroup request and response APIs","ref":"KafkaEx.Protocol.JoinGroup.html"},{"type":"function","title":"KafkaEx.Protocol.JoinGroup.create_request/3","doc":"","ref":"KafkaEx.Protocol.JoinGroup.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.JoinGroup.parse_response/1","doc":"","ref":"KafkaEx.Protocol.JoinGroup.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.LeaveGroup","doc":"","ref":"KafkaEx.Protocol.LeaveGroup.html"},{"type":"function","title":"KafkaEx.Protocol.LeaveGroup.create_request/3","doc":"","ref":"KafkaEx.Protocol.LeaveGroup.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.LeaveGroup.parse_response/1","doc":"","ref":"KafkaEx.Protocol.LeaveGroup.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.Metadata","doc":"Implementation of the Kafka Heartbeat request and response APIs","ref":"KafkaEx.Protocol.Metadata.html"},{"type":"function","title":"KafkaEx.Protocol.Metadata.api_version/1","doc":"","ref":"KafkaEx.Protocol.Metadata.html#api_version/1"},{"type":"function","title":"KafkaEx.Protocol.Metadata.create_request/4","doc":"","ref":"KafkaEx.Protocol.Metadata.html#create_request/4"},{"type":"function","title":"KafkaEx.Protocol.Metadata.parse_response/1","doc":"","ref":"KafkaEx.Protocol.Metadata.html#parse_response/1"},{"type":"function","title":"KafkaEx.Protocol.Metadata.parse_response/2","doc":"","ref":"KafkaEx.Protocol.Metadata.html#parse_response/2"},{"type":"module","title":"KafkaEx.Protocol.Offset","doc":"Implementation of the Kafka Offset request and response APIs","ref":"KafkaEx.Protocol.Offset.html"},{"type":"function","title":"KafkaEx.Protocol.Offset.create_request/5","doc":"","ref":"KafkaEx.Protocol.Offset.html#create_request/5"},{"type":"function","title":"KafkaEx.Protocol.Offset.parse_response/1","doc":"","ref":"KafkaEx.Protocol.Offset.html#parse_response/1"},{"type":"function","title":"KafkaEx.Protocol.Offset.parse_time/1","doc":"","ref":"KafkaEx.Protocol.Offset.html#parse_time/1"},{"type":"module","title":"KafkaEx.Protocol.OffsetCommit","doc":"Implementation of the Kafka OffsetCommit request and response APIs","ref":"KafkaEx.Protocol.OffsetCommit.html"},{"type":"function","title":"KafkaEx.Protocol.OffsetCommit.create_request/3","doc":"","ref":"KafkaEx.Protocol.OffsetCommit.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.OffsetCommit.parse_response/1","doc":"","ref":"KafkaEx.Protocol.OffsetCommit.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.OffsetFetch","doc":"Implementation of the Kafka OffsetFetch request and response APIs","ref":"KafkaEx.Protocol.OffsetFetch.html"},{"type":"function","title":"KafkaEx.Protocol.OffsetFetch.create_request/3","doc":"","ref":"KafkaEx.Protocol.OffsetFetch.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.OffsetFetch.parse_partitions/4","doc":"","ref":"KafkaEx.Protocol.OffsetFetch.html#parse_partitions/4"},{"type":"function","title":"KafkaEx.Protocol.OffsetFetch.parse_response/1","doc":"","ref":"KafkaEx.Protocol.OffsetFetch.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.Produce","doc":"Implementation of the Kafka Produce request and response APIs","ref":"KafkaEx.Protocol.Produce.html"},{"type":"function","title":"KafkaEx.Protocol.Produce.create_request/3","doc":"","ref":"KafkaEx.Protocol.Produce.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.Produce.parse_partitions/4","doc":"","ref":"KafkaEx.Protocol.Produce.html#parse_partitions/4"},{"type":"function","title":"KafkaEx.Protocol.Produce.parse_response/1","doc":"","ref":"KafkaEx.Protocol.Produce.html#parse_response/1"},{"type":"module","title":"KafkaEx.Protocol.Produce.Message","doc":"- key: is used for partition assignment, can be nil, when none is provided\nit is defaulted to nil\n- value: is the message to be written to Kafka logs.\n- headers: is a list of keys and values that adds metadata to messages\n- timestamp: timestamp (`kafka_version: \"kayrock\"` ONLY)","ref":"KafkaEx.Protocol.Produce.Message.html"},{"type":"type","title":"KafkaEx.Protocol.Produce.Message.t/0","doc":"","ref":"KafkaEx.Protocol.Produce.Message.html#t:t/0"},{"type":"module","title":"KafkaEx.Protocol.Produce.Request","doc":"- required_acks: indicates how many acknowledgements the servers should\nreceive before responding to the request. If it is 0 the server will not\nsend any response (this is the only case where the server will not reply\nto a request). If it is 1, the server will wait the data is written to the\nlocal log before sending a response. If it is -1 the server will block until\nthe message is committed by all in sync replicas before sending a response.\nFor any number > 1 the server will block waiting for this number of\nacknowledgements to occur (but the server will never wait for more\nacknowledgements than there are in-sync replicas), default is 0\n- timeout: provides a maximum time in milliseconds the server can await the\nreceipt of the number of acknowledgements in RequiredAcks, default is 100\nmilliseconds","ref":"KafkaEx.Protocol.Produce.Request.html"},{"type":"type","title":"KafkaEx.Protocol.Produce.Request.t/0","doc":"","ref":"KafkaEx.Protocol.Produce.Request.html#t:t/0"},{"type":"module","title":"KafkaEx.Protocol.SyncGroup","doc":"Implementation of the Kafka SyncGroup request and response APIs","ref":"KafkaEx.Protocol.SyncGroup.html"},{"type":"function","title":"KafkaEx.Protocol.SyncGroup.create_request/3","doc":"","ref":"KafkaEx.Protocol.SyncGroup.html#create_request/3"},{"type":"function","title":"KafkaEx.Protocol.SyncGroup.parse_response/1","doc":"","ref":"KafkaEx.Protocol.SyncGroup.html#parse_response/1"},{"type":"behaviour","title":"KafkaEx.Server","doc":"Defines the KafkaEx.Server behavior that all Kafka API servers must implement, this module also provides some common callback functions that are injected into the servers that `use` it.","ref":"KafkaEx.Server.html"},{"type":"callback","title":"KafkaEx.Server.kafka_server_api_versions/1","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_api_versions/1"},{"type":"callback","title":"KafkaEx.Server.kafka_server_consumer_group/1","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_consumer_group/1"},{"type":"callback","title":"KafkaEx.Server.kafka_server_consumer_group_metadata/1","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_consumer_group_metadata/1"},{"type":"callback","title":"KafkaEx.Server.kafka_server_create_topics/3","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_create_topics/3"},{"type":"callback","title":"KafkaEx.Server.kafka_server_delete_topics/3","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_delete_topics/3"},{"type":"callback","title":"KafkaEx.Server.kafka_server_fetch/2","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_fetch/2"},{"type":"callback","title":"KafkaEx.Server.kafka_server_heartbeat/3","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_heartbeat/3"},{"type":"callback","title":"KafkaEx.Server.kafka_server_init/1","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_init/1"},{"type":"callback","title":"KafkaEx.Server.kafka_server_join_group/3","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_join_group/3"},{"type":"callback","title":"KafkaEx.Server.kafka_server_leave_group/3","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_leave_group/3"},{"type":"callback","title":"KafkaEx.Server.kafka_server_metadata/2","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_metadata/2"},{"type":"callback","title":"KafkaEx.Server.kafka_server_offset/4","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_offset/4"},{"type":"callback","title":"KafkaEx.Server.kafka_server_offset_commit/2","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_offset_commit/2"},{"type":"callback","title":"KafkaEx.Server.kafka_server_offset_fetch/2","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_offset_fetch/2"},{"type":"callback","title":"KafkaEx.Server.kafka_server_produce/2","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_produce/2"},{"type":"callback","title":"KafkaEx.Server.kafka_server_sync_group/3","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_sync_group/3"},{"type":"callback","title":"KafkaEx.Server.kafka_server_update_consumer_metadata/1","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_update_consumer_metadata/1"},{"type":"callback","title":"KafkaEx.Server.kafka_server_update_metadata/1","doc":"","ref":"KafkaEx.Server.html#c:kafka_server_update_metadata/1"},{"type":"module","title":"KafkaEx.Server0P10AndLater","doc":"Implements KafkaEx.Server behaviors for Kafka 0.10.1 API.","ref":"KafkaEx.Server0P10AndLater.html"},{"type":"function","title":"KafkaEx.Server0P10AndLater.broker_for_consumer_group_with_update/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#broker_for_consumer_group_with_update/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.consumer_group?/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#consumer_group?/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.handle_call/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#handle_call/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.handle_info/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#handle_info/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.init/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#init/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_api_versions/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_api_versions/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_consumer_group/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_consumer_group/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_consumer_group_metadata/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_consumer_group_metadata/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_create_topics/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_create_topics/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_delete_topics/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_delete_topics/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_fetch/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_fetch/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_heartbeat/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_heartbeat/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_init/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_init/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_join_group/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_join_group/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_leave_group/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_leave_group/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_metadata/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_metadata/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_offset/4","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_offset/4"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_offset_commit/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_offset_commit/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_offset_fetch/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_offset_fetch/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_produce/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_produce/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_produce_send_request/4","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_produce_send_request/4"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_sync_group/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_sync_group/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_update_consumer_metadata/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_update_consumer_metadata/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.kafka_server_update_metadata/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#kafka_server_update_metadata/1"},{"type":"function","title":"KafkaEx.Server0P10AndLater.retrieve_metadata/5","doc":"","ref":"KafkaEx.Server0P10AndLater.html#retrieve_metadata/5"},{"type":"function","title":"KafkaEx.Server0P10AndLater.retrieve_metadata/7","doc":"","ref":"KafkaEx.Server0P10AndLater.html#retrieve_metadata/7"},{"type":"function","title":"KafkaEx.Server0P10AndLater.retrieve_metadata_with_version/7","doc":"","ref":"KafkaEx.Server0P10AndLater.html#retrieve_metadata_with_version/7"},{"type":"function","title":"KafkaEx.Server0P10AndLater.start_link/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#start_link/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.terminate/2","doc":"","ref":"KafkaEx.Server0P10AndLater.html#terminate/2"},{"type":"function","title":"KafkaEx.Server0P10AndLater.update_consumer_metadata/3","doc":"","ref":"KafkaEx.Server0P10AndLater.html#update_consumer_metadata/3"},{"type":"function","title":"KafkaEx.Server0P10AndLater.update_metadata/1","doc":"","ref":"KafkaEx.Server0P10AndLater.html#update_metadata/1"},{"type":"module","title":"KafkaEx.Server0P8P0","doc":"Implements KafkaEx.Server behaviors for Kafka >= 0.8.0 < 0.8.2 API.","ref":"KafkaEx.Server0P8P0.html"},{"type":"function","title":"KafkaEx.Server0P8P0.handle_call/3","doc":"","ref":"KafkaEx.Server0P8P0.html#handle_call/3"},{"type":"function","title":"KafkaEx.Server0P8P0.handle_info/2","doc":"","ref":"KafkaEx.Server0P8P0.html#handle_info/2"},{"type":"function","title":"KafkaEx.Server0P8P0.init/1","doc":"","ref":"KafkaEx.Server0P8P0.html#init/1"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_api_versions/1","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_api_versions/1"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_consumer_group/1","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_consumer_group/1"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_consumer_group_metadata/1","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_consumer_group_metadata/1"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_create_topics/3","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_create_topics/3"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_delete_topics/3","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_delete_topics/3"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_fetch/2","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_fetch/2"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_heartbeat/3","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_heartbeat/3"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_init/1","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_init/1"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_join_group/3","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_join_group/3"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_leave_group/3","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_leave_group/3"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_metadata/2","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_metadata/2"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_offset/4","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_offset/4"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_offset_commit/2","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_offset_commit/2"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_offset_fetch/2","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_offset_fetch/2"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_produce/2","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_produce/2"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_produce_send_request/4","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_produce_send_request/4"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_sync_group/3","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_sync_group/3"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_update_consumer_metadata/1","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_update_consumer_metadata/1"},{"type":"function","title":"KafkaEx.Server0P8P0.kafka_server_update_metadata/1","doc":"","ref":"KafkaEx.Server0P8P0.html#kafka_server_update_metadata/1"},{"type":"function","title":"KafkaEx.Server0P8P0.retrieve_metadata/5","doc":"","ref":"KafkaEx.Server0P8P0.html#retrieve_metadata/5"},{"type":"function","title":"KafkaEx.Server0P8P0.retrieve_metadata/7","doc":"","ref":"KafkaEx.Server0P8P0.html#retrieve_metadata/7"},{"type":"function","title":"KafkaEx.Server0P8P0.retrieve_metadata_with_version/7","doc":"","ref":"KafkaEx.Server0P8P0.html#retrieve_metadata_with_version/7"},{"type":"function","title":"KafkaEx.Server0P8P0.start_link/2","doc":"","ref":"KafkaEx.Server0P8P0.html#start_link/2"},{"type":"function","title":"KafkaEx.Server0P8P0.terminate/2","doc":"","ref":"KafkaEx.Server0P8P0.html#terminate/2"},{"type":"function","title":"KafkaEx.Server0P8P0.update_metadata/1","doc":"","ref":"KafkaEx.Server0P8P0.html#update_metadata/1"},{"type":"module","title":"KafkaEx.Server0P8P2","doc":"Implements KafkaEx.Server behaviors for kafka >= 0.8.2 < 0.9.0 API.","ref":"KafkaEx.Server0P8P2.html"},{"type":"function","title":"KafkaEx.Server0P8P2.consumer_group?/1","doc":"","ref":"KafkaEx.Server0P8P2.html#consumer_group?/1"},{"type":"function","title":"KafkaEx.Server0P8P2.consumer_group_if_auto_commit?/2","doc":"","ref":"KafkaEx.Server0P8P2.html#consumer_group_if_auto_commit?/2"},{"type":"function","title":"KafkaEx.Server0P8P2.handle_call/3","doc":"","ref":"KafkaEx.Server0P8P2.html#handle_call/3"},{"type":"function","title":"KafkaEx.Server0P8P2.handle_info/2","doc":"","ref":"KafkaEx.Server0P8P2.html#handle_info/2"},{"type":"function","title":"KafkaEx.Server0P8P2.init/1","doc":"","ref":"KafkaEx.Server0P8P2.html#init/1"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_api_versions/1","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_api_versions/1"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_consumer_group/1","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_consumer_group/1"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_consumer_group_metadata/1","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_consumer_group_metadata/1"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_create_topics/3","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_create_topics/3"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_delete_topics/3","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_delete_topics/3"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_fetch/2","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_fetch/2"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_heartbeat/3","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_heartbeat/3"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_init/1","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_init/1"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_join_group/3","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_join_group/3"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_leave_group/3","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_leave_group/3"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_metadata/2","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_metadata/2"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_offset/4","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_offset/4"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_offset_commit/2","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_offset_commit/2"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_offset_fetch/2","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_offset_fetch/2"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_produce/2","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_produce/2"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_produce_send_request/4","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_produce_send_request/4"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_sync_group/3","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_sync_group/3"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_update_consumer_metadata/1","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_update_consumer_metadata/1"},{"type":"function","title":"KafkaEx.Server0P8P2.kafka_server_update_metadata/1","doc":"","ref":"KafkaEx.Server0P8P2.html#kafka_server_update_metadata/1"},{"type":"function","title":"KafkaEx.Server0P8P2.retrieve_metadata/5","doc":"","ref":"KafkaEx.Server0P8P2.html#retrieve_metadata/5"},{"type":"function","title":"KafkaEx.Server0P8P2.retrieve_metadata/7","doc":"","ref":"KafkaEx.Server0P8P2.html#retrieve_metadata/7"},{"type":"function","title":"KafkaEx.Server0P8P2.retrieve_metadata_with_version/7","doc":"","ref":"KafkaEx.Server0P8P2.html#retrieve_metadata_with_version/7"},{"type":"function","title":"KafkaEx.Server0P8P2.start_link/2","doc":"","ref":"KafkaEx.Server0P8P2.html#start_link/2"},{"type":"function","title":"KafkaEx.Server0P8P2.terminate/2","doc":"","ref":"KafkaEx.Server0P8P2.html#terminate/2"},{"type":"function","title":"KafkaEx.Server0P8P2.update_consumer_metadata/3","doc":"","ref":"KafkaEx.Server0P8P2.html#update_consumer_metadata/3"},{"type":"function","title":"KafkaEx.Server0P8P2.update_metadata/1","doc":"","ref":"KafkaEx.Server0P8P2.html#update_metadata/1"},{"type":"module","title":"KafkaEx.Server0P9P0","doc":"Implements KafkaEx.Server behaviors for Kafka >= 0.9.0 < 0.10.1 API.","ref":"KafkaEx.Server0P9P0.html"},{"type":"function","title":"KafkaEx.Server0P9P0.broker_for_consumer_group_with_update/2","doc":"","ref":"KafkaEx.Server0P9P0.html#broker_for_consumer_group_with_update/2"},{"type":"function","title":"KafkaEx.Server0P9P0.consumer_group?/1","doc":"","ref":"KafkaEx.Server0P9P0.html#consumer_group?/1"},{"type":"function","title":"KafkaEx.Server0P9P0.handle_call/3","doc":"","ref":"KafkaEx.Server0P9P0.html#handle_call/3"},{"type":"function","title":"KafkaEx.Server0P9P0.handle_info/2","doc":"","ref":"KafkaEx.Server0P9P0.html#handle_info/2"},{"type":"function","title":"KafkaEx.Server0P9P0.init/1","doc":"","ref":"KafkaEx.Server0P9P0.html#init/1"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_api_versions/1","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_api_versions/1"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_consumer_group/1","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_consumer_group/1"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_consumer_group_metadata/1","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_consumer_group_metadata/1"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_create_topics/3","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_create_topics/3"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_delete_topics/3","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_delete_topics/3"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_fetch/2","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_fetch/2"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_heartbeat/3","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_heartbeat/3"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_init/1","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_init/1"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_join_group/3","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_join_group/3"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_leave_group/3","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_leave_group/3"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_metadata/2","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_metadata/2"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_offset/4","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_offset/4"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_offset_commit/2","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_offset_commit/2"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_offset_fetch/2","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_offset_fetch/2"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_produce/2","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_produce/2"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_produce_send_request/4","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_produce_send_request/4"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_sync_group/3","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_sync_group/3"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_update_consumer_metadata/1","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_update_consumer_metadata/1"},{"type":"function","title":"KafkaEx.Server0P9P0.kafka_server_update_metadata/1","doc":"","ref":"KafkaEx.Server0P9P0.html#kafka_server_update_metadata/1"},{"type":"function","title":"KafkaEx.Server0P9P0.retrieve_metadata/5","doc":"","ref":"KafkaEx.Server0P9P0.html#retrieve_metadata/5"},{"type":"function","title":"KafkaEx.Server0P9P0.retrieve_metadata/7","doc":"","ref":"KafkaEx.Server0P9P0.html#retrieve_metadata/7"},{"type":"function","title":"KafkaEx.Server0P9P0.retrieve_metadata_with_version/7","doc":"","ref":"KafkaEx.Server0P9P0.html#retrieve_metadata_with_version/7"},{"type":"function","title":"KafkaEx.Server0P9P0.start_link/2","doc":"","ref":"KafkaEx.Server0P9P0.html#start_link/2"},{"type":"function","title":"KafkaEx.Server0P9P0.terminate/2","doc":"","ref":"KafkaEx.Server0P9P0.html#terminate/2"},{"type":"function","title":"KafkaEx.Server0P9P0.update_consumer_metadata/3","doc":"","ref":"KafkaEx.Server0P9P0.html#update_consumer_metadata/3"},{"type":"function","title":"KafkaEx.Server0P9P0.update_metadata/1","doc":"","ref":"KafkaEx.Server0P9P0.html#update_metadata/1"},{"type":"module","title":"KafkaEx.Socket","doc":"This module handle all socket related operations.","ref":"KafkaEx.Socket.html"},{"type":"function","title":"KafkaEx.Socket.close/1","doc":"Closes the socket.\n\nFor more information, see `:ssl.close/1` for ssl or `:gen_tcp.send/1` for non ssl.","ref":"KafkaEx.Socket.html#close/1"},{"type":"function","title":"KafkaEx.Socket.create/4","doc":"Creates a socket.\n\nFor more information about the available options, see `:ssl.connect/3` for ssl\nor `:gen_tcp.connect/3` for non ssl.","ref":"KafkaEx.Socket.html#create/4"},{"type":"function","title":"KafkaEx.Socket.info/1","doc":"Returns the information about the socket.\n\nFor more information, see `Port.info`","ref":"KafkaEx.Socket.html#info/1"},{"type":"function","title":"KafkaEx.Socket.open?/1","doc":"Returns true if the socket is open","ref":"KafkaEx.Socket.html#open?/1"},{"type":"function","title":"KafkaEx.Socket.recv/2","doc":"Receives data from the socket.\n\nFor more information, see `:ssl.recv/2` for ssl or `:gen_tcp.recv/2` for non ssl.","ref":"KafkaEx.Socket.html#recv/2"},{"type":"function","title":"KafkaEx.Socket.recv/3","doc":"","ref":"KafkaEx.Socket.html#recv/3"},{"type":"function","title":"KafkaEx.Socket.send/2","doc":"Sends data over the socket.\n\nIt handles both, SSL and non SSL sockets.\n\nFor more information, see `:ssl.send/2` for ssl or `:gen_tcp.send/2` for non ssl.","ref":"KafkaEx.Socket.html#send/2"},{"type":"function","title":"KafkaEx.Socket.setopts/2","doc":"Set options to the socket.\n\nFor more information, see `:ssl.setopts/2` for ssl or `:inet.setopts/2` for non ssl.","ref":"KafkaEx.Socket.html#setopts/2"},{"type":"type","title":"KafkaEx.Socket.t/0","doc":"","ref":"KafkaEx.Socket.html#t:t/0"},{"type":"module","title":"KafkaEx.Utils.Murmur","doc":"Utility module that provides Murmur hashing algorithm.","ref":"KafkaEx.Utils.Murmur.html"},{"type":"function","title":"KafkaEx.Utils.Murmur.murmur2/1","doc":"Calculates murmur2 hash for given binary","ref":"KafkaEx.Utils.Murmur.html#murmur2/1"},{"type":"function","title":"KafkaEx.Utils.Murmur.umurmur2/1","doc":"Calculates murmur2 hash for given binary as unsigned 32-bit integer","ref":"KafkaEx.Utils.Murmur.html#umurmur2/1"},{"type":"function","title":"KafkaEx.Utils.Murmur.umurmur2_legacy/1","doc":"Calculates murmur2 hash for given binary as unsigned 32-bit integer\n\nThis is to support the legacy default partitioner implementation.","ref":"KafkaEx.Utils.Murmur.html#umurmur2_legacy/1"},{"type":"exception","title":"KafkaEx.ConsumerGroupRequiredError","doc":"","ref":"KafkaEx.ConsumerGroupRequiredError.html"},{"type":"exception","title":"KafkaEx.InvalidConsumerGroupError","doc":"","ref":"KafkaEx.InvalidConsumerGroupError.html"},{"type":"exception","title":"KafkaEx.TimestampNotSupportedError","doc":"","ref":"KafkaEx.TimestampNotSupportedError.html"},{"type":"extras","title":"KafkaEx","doc":"KafkaEx\n========\n\n[![CI Tests](https://github.com/kafkaex/kafka_ex/actions/workflows/test.yml/badge.svg)](https://github.com/kafkaex/kafka_ex/actions/workflows/test.yml)\n[![CI Checks](https://github.com/kafkaex/kafka_ex/actions/workflows/checks.yml/badge.svg)](https://github.com/kafkaex/kafka_ex/actions/workflows/checks.yml)\n[![Coverage Status](https://coveralls.io/repos/github/kafkaex/kafka_ex/badge.svg?branch=master)](https://coveralls.io/github/kafkaex/kafka_ex?branch=master)\n[![Hex.pm version](https://img.shields.io/hexpm/v/kafka_ex.svg?style=flat-square)](https://hex.pm/packages/kafka_ex)\n[![Hex.pm downloads](https://img.shields.io/hexpm/dt/kafka_ex.svg?style=flat-square)](https://hex.pm/packages/kafka_ex)\n[![License](https://img.shields.io/hexpm/l/kafka_ex.svg?style=flat-square)](https://hex.pm/packages/kafka_ex)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](http://hexdocs.pm/kafka_ex/)\n\nKafkaEx is an Elixir client for [Apache Kafka](http://kafka.apache.org/) with\nsupport for Kafka versions 0.8.0 and newer. KafkaEx requires Elixir 1.6+ and\nErlang OTP 19+.\n\nSee [http://hexdocs.pm/kafka_ex/](http://hexdocs.pm/kafka_ex/) for\ndocumentation,\n [https://github.com/kafkaex/kafka_ex/](https://github.com/kafkaex/kafka_ex/)\n for code.\n\nKafkaEx supports the following Kafka features:\n\n*   Broker and Topic Metadata\n*   Produce Messages\n*   Fetch Messages\n*   Message Compression with Snappy and gzip\n*   Offset Management (fetch / commit / autocommit)\n*   Consumer Groups\n*   Topics Management (create / delete)\n\nSee [Kafka Protocol Documentation](http://kafka.apache.org/protocol.html) and\n [A Guide to the Kafka Protocol](https://cwiki.apache.org/confluence/display/KAFKA/A+Guide+To+The+Kafka+Protocol)\nfor details of these features.","ref":"readme.html"},{"type":"extras","title":"IMPORTANT - Kayrock and The Future of KafkaEx - KafkaEx","doc":"TL;DR:\n\n*   This is new implementation and we need people to test it!\n*   Set `kafka_version: \"kayrock\"` to use the new client implementation.\n*   The new client should be compatible with existing code when used this way.\n*   Many functions now support an `api_version` parameter, see below for details,\n    e.g., how to store offsets in Kafka instead of Zookeeper.\n*   Version 1.0 of KafkaEx will be based on Kayrock and have a cleaner API - you\n    can start testing this API by using modules from the `KafkaEx.New` namespace.\n    See below for details.\n*   Version 0.11.0+ of KafkaEx is required to use Kayrock.\n\nTo support some oft-requested features (offset storage in Kafka, message\ntimestamps), we have integrated KafkaEx with\n[Kayrock](https://github.com/dantswain/kayrock) which is a library that handles\nserialization and deserialization of the Kafka message protocol in a way that\ncan grow as Kafka does.\n\nUnfortunately, the existing KafkaEx API is built in such a way that it doesn't\neasily support this growth.  This, combined with a number of other existing\nwarts in the current API, has led us to the conclusion that v1.0 of KafkaEx\nshould have a new and cleaner API.\n\nThe path we have planned to get to v1.0 is:\n\n1.  Add a Kayrock compatibility layer for the existing KafkaEx API (DONE, not released).\n2.  Expose Kayrock's API versioning through a select handful of KafkaEx API\n    functions so that users can get access to the most-requested features (e.g.,\n    offset storage in Kafka and message timestamps) (DONE, not released).\n3.  Begin designing and implementing the new API in parallel in the `KafkaEx.New`\n    namespace (EARLY PROGRESS).\n4.  Incrementally release the new API alongside the legacy API so that early\n    adopters can test it.\n5.  Once the new API is complete and stable, move it to the `KafkaEx` namespace\n    (i.e., drop the `New` part) and it will replace the legacy API.  This will be\n    released as v1.0.\n\nUsers of KafkaEx can help a lot by testing the new code. At first, we need\npeople to test the Kayrock-based client using compatibility mode. You can do\nthis by simply setting `kafka_version: \"kayrock\"` in your configuration. That\nshould be all you need to change. If you want to test new features enabled by\n`api_versions` options then that is also very valuable to us (see below for\nlinks to details). Then, as work on the new API ramps up, users can\ncontribute feedback to pull requests (or even contribute pull requests!) and\ntest out the new API as it becomes available.\n\nFor more information on using the Kayrock-based client, see\n\n*   Github: [kayrock.md](https://github.com/kafkaex/kafka_ex/blob/master/kayrock.md)\n*   HexDocs: [kayrock-based client](kayrock.html)\n\nFor more information on the v1.0 API, see\n\n*   Github:\n    [new_api.md](https://github.com/kafkaex/kafka_ex/blob/master/new_api.md)\n*   HexDocs: [New API](new_api.html)","ref":"readme.html#important-kayrock-and-the-future-of-kafkaex"},{"type":"extras","title":"Using KafkaEx in an Elixir project - KafkaEx","doc":"The standard approach for adding dependencies to an Elixir application applies:\nadd KafkaEx to the deps list in your project's mix.exs file.\nYou may also optionally add\n[snappyer](https://hex.pm/packages/snappyer) (required\nonly if you want to use snappy compression).\n\n```elixir\n# mix.exs\ndefmodule MyApp.Mixfile do\n  # ...\n\n  defp deps do\n    [\n      # add to your existing deps\n      {:kafka_ex, \"~> 0.11\"},\n      # If using snappy-erlang-nif (snappy) compression\n      {:snappy, git: \"https://github.com/fdmanana/snappy-erlang-nif\"}\n      # if using snappyer (snappy) compression\n      {:snappyer, \"~> 1.2\"}\n    ]\n  end\nend\n```\n\nThen run `mix deps.get` to fetch dependencies.","ref":"readme.html#using-kafkaex-in-an-elixir-project"},{"type":"extras","title":"Configuration - KafkaEx","doc":"See [config/config.exs](https://github.com/kafkaex/kafka_ex/blob/master/config/config.exs)\nor [KafkaEx.Config](https://hexdocs.pm/kafka_ex/KafkaEx.Config.html)\nfor a description of configuration variables, including the Kafka broker list\n and default consumer group.\n\nYou can also override options when creating a worker, see below.","ref":"readme.html#configuration"},{"type":"extras","title":"Timeouts with SSL - KafkaEx","doc":"When using certain versions of OTP,\n[random timeouts can occur if using SSL](https://github.com/kafkaex/kafka_ex/issues/389).\n\nImpacted versions:\n\n*   OTP 21.3.8.1 -> 21.3.8.14\n*   OTP 22.1 -> 22.3.1\n\nUpgrade respectively to 21.3.8.15 or 22.3.2 to solve this.","ref":"readme.html#timeouts-with-ssl"},{"type":"extras","title":"Usage Examples - KafkaEx","doc":"#","ref":"readme.html#usage-examples"},{"type":"extras","title":"Consumer Groups - KafkaEx","doc":"To use a consumer group, first implement a handler module using\n`KafkaEx.GenConsumer`.\n\n```elixir\ndefmodule ExampleGenConsumer do\n  use KafkaEx.GenConsumer\n\n  alias KafkaEx.Protocol.Fetch.Message\n\n  require Logger\n\n  # note - messages are delivered in batches\n  def handle_message_set(message_set, state) do\n    for %Message{value: message} <- message_set do\n      Logger.debug(fn -> \"message: \" <> inspect(message) end)\n    end\n    {:async_commit, state}\n  end\nend\n```\n\nThen add a `KafkaEx.ConsumerGroup` to your application's supervision\ntree and configure it to use the implementation module.\n\nSee the `KafkaEx.GenConsumer` and `KafkaEx.ConsumerGroup` documentation for\ndetails.\n\n#","ref":"readme.html#consumer-groups"},{"type":"extras","title":"Create a KafkaEx Worker - KafkaEx","doc":"KafkaEx worker processes manage the state of the connection to the Kafka broker.\n\n```elixir\niex> KafkaEx.create_worker(:pr) # where :pr is the process name of the created worker\n{:ok, #PID<0.171.0>}\n```\n\nWith custom options:\n\n```elixir\niex> uris = [{\"localhost\", 9092}, {\"localhost\", 9093}, {\"localhost\", 9094}]\n[{\"localhost\", 9092}, {\"localhost\", 9093}, {\"localhost\", 9094}]\niex> KafkaEx.create_worker(:pr, [uris: uris, consumer_group: \"kafka_ex\", consumer_group_update_interval: 100])\n{:ok, #PID<0.172.0>}\n```\n\n#","ref":"readme.html#create-a-kafkaex-worker"},{"type":"extras","title":"Create an unnamed KafkaEx worker - KafkaEx","doc":"You may find you want to create many workers, say in conjunction with\na `poolboy` pool. In this scenario you usually won't want to name these worker processes.\n\nTo create an unnamed worked with `create_worker`:\n```elixir\niex> KafkaEx.create_worker(:no_name) # indicates to the server process not to name the process\n{:ok, #PID<0.171.0>}\n```\n\n#","ref":"readme.html#create-an-unnamed-kafkaex-worker"},{"type":"extras","title":"Use KafkaEx with a pooling library - KafkaEx","doc":"Note that KafkaEx has a supervisor to manage its workers. If you are using Poolboy or a similar\nlibrary, you will want to manually create a worker so that it is not supervised by `KafkaEx.Supervisor`.\nTo do this, you will need to call:\n\n```elixir\nGenServer.start_link(KafkaEx.Config.server_impl,\n  [\n    [uris: KafkaEx.Config.brokers(),\n     consumer_group: Application.get_env(:kafka_ex, :consumer_group)],\n    :no_name\n  ]\n)\n```\n\nAlternatively, you can call\n\n```\nKafkaEx.start_link_worker(:no_name)\n```\n\n#","ref":"readme.html#use-kafkaex-with-a-pooling-library"},{"type":"extras","title":"Retrieve kafka metadata - KafkaEx","doc":"For all metadata\n\n```elixir\niex> KafkaEx.metadata\n%KafkaEx.Protocol.Metadata.Response{brokers: [%KafkaEx.Protocol.Metadata.Broker{host:\n \"192.168.59.103\",\n   node_id: 49162, port: 49162, socket: nil}],\n topic_metadatas: [%KafkaEx.Protocol.Metadata.TopicMetadata{error_code: :no_error,\n   partition_metadatas: [%KafkaEx.Protocol.Metadata.PartitionMetadata{error_code: :no_error,\n     isrs: [49162], leader: 49162, partition_id: 0, replicas: [49162]}],\n   topic: \"LRCYFQDVWUFEIUCCTFGP\"},\n  %KafkaEx.Protocol.Metadata.TopicMetadata{error_code: :no_error,\n   partition_metadatas: [%KafkaEx.Protocol.Metadata.PartitionMetadata{error_code: :no_error,\n     isrs: [49162], leader: 49162, partition_id: 0, replicas: [49162]}],\n   topic: \"JSIMKCLQYTWXMSIGESYL\"},\n  %KafkaEx.Protocol.Metadata.TopicMetadata{error_code: :no_error,\n   partition_metadatas: [%KafkaEx.Protocol.Metadata.PartitionMetadata{error_code: :no_error,\n     isrs: [49162], leader: 49162, partition_id: 0, replicas: [49162]}],\n   topic: \"SCFRRXXLDFPOWSPQQMSD\"},\n  %KafkaEx.Protocol.Metadata.TopicMetadata{error_code: :no_error,\n...\n```\n\nFor a specific topic\n\n```elixir\niex> KafkaEx.metadata(topic: \"foo\")\n%KafkaEx.Protocol.Metadata.Response{brokers: [%KafkaEx.Protocol.Metadata.Broker{host: \"192.168.59.103\",\n   node_id: 49162, port: 49162, socket: nil}],\n topic_metadatas: [%KafkaEx.Protocol.Metadata.TopicMetadata{error_code: :no_error,\n   partition_metadatas: [%KafkaEx.Protocol.Metadata.PartitionMetadata{error_code: :no_error,\n     isrs: [49162], leader: 49162, partition_id: 0, replicas: [49162]}],\n   topic: \"foo\"}]}\n```\n\n#","ref":"readme.html#retrieve-kafka-metadata"},{"type":"extras","title":"Retrieve offset from a particular time - KafkaEx","doc":"Kafka will get the starting offset of the log segment that is created no later than the given timestamp. Due to this, and since the offset request is served only at segment granularity, the offset fetch request returns less accurate results for larger segment sizes.\n\n```elixir\niex> KafkaEx.offset(\"foo\", 0, {{2015, 3, 29}, {23, 56, 40}}) # Note that the time specified should match/be ahead of time on the server that kafka runs\n[%KafkaEx.Protocol.Offset.Response{partition_offsets: [%{error_code: :no_error, offset: [256], partition: 0}], topic: \"foo\"}]\n```\n\n#","ref":"readme.html#retrieve-offset-from-a-particular-time"},{"type":"extras","title":"Retrieve the latest offset - KafkaEx","doc":"```elixir\niex> KafkaEx.latest_offset(\"foo\", 0) # where 0 is the partition\n[%KafkaEx.Protocol.Offset.Response{partition_offsets: [%{error_code: :no_error, offset: [16], partition: 0}], topic: \"foo\"}]\n```\n\n#","ref":"readme.html#retrieve-the-latest-offset"},{"type":"extras","title":"Retrieve the earliest offset - KafkaEx","doc":"```elixir\niex> KafkaEx.earliest_offset(\"foo\", 0) # where 0 is the partition\n[%KafkaEx.Protocol.Offset.Response{partition_offsets: [%{error_code: :no_error, offset: [0], partition: 0}], topic: \"foo\"}]\n```\n\n#","ref":"readme.html#retrieve-the-earliest-offset"},{"type":"extras","title":"Fetch kafka logs - KafkaEx","doc":"**NOTE** You must pass `auto_commit: false` in the options for `fetch/3` when using Kafka < 0.8.2 or when using `:no_consumer_group`.\n\n```elixir\niex> KafkaEx.fetch(\"foo\", 0, offset: 5) # where 0 is the partition and 5 is the offset we want to start fetching from\n[%KafkaEx.Protocol.Fetch.Response{partitions: [%{error_code: :no_error,\n     hw_mark_offset: 115,\n     message_set: [\n      %KafkaEx.Protocol.Fetch.Message{attributes: 0, crc: 4264455069, key: nil, offset: 5, value: \"hey\"},\n      %KafkaEx.Protocol.Fetch.Message{attributes: 0, crc: 4264455069, key: nil, offset: 6, value: \"hey\"},\n      %KafkaEx.Protocol.Fetch.Message{attributes: 0, crc: 4264455069, key: nil, offset: 7, value: \"hey\"},\n      %KafkaEx.Protocol.Fetch.Message{attributes: 0, crc: 4264455069, key: nil, offset: 8, value: \"hey\"},\n      %KafkaEx.Protocol.Fetch.Message{attributes: 0, crc: 4264455069, key: nil, offset: 9, value: \"hey\"}\n...], partition: 0}], topic: \"foo\"}]\n```\n\n#","ref":"readme.html#fetch-kafka-logs"},{"type":"extras","title":"Produce kafka logs - KafkaEx","doc":"```elixir\niex> KafkaEx.produce(\"foo\", 0, \"hey\") # where \"foo\" is the topic and \"hey\" is the message\n:ok\n```\n\n#","ref":"readme.html#produce-kafka-logs"},{"type":"extras","title":"Stream kafka logs - KafkaEx","doc":"See the `KafkaEx.stream/3` documentation for details on streaming.\n\n```elixir\niex> KafkaEx.produce(\"foo\", 0, \"hey\")\n:ok\niex> KafkaEx.produce(\"foo\", 0, \"hi\")\n:ok\niex> KafkaEx.stream(\"foo\", 0, offset: 0) |> Enum.take(2)\n[%{attributes: 0, crc: 4264455069, key: nil, offset: 0, value: \"hey\"},\n %{attributes: 0, crc: 4251893211, key: nil, offset: 1, value: \"hi\"}]\n```\n\nFor Kafka < 0.8.2 the `stream/3` requires `auto_commit: false`\n\n```elixir\niex> KafkaEx.stream(\"foo\", 0, offset: 0, auto_commit: false) |> Enum.take(2)\n```\n\n#","ref":"readme.html#stream-kafka-logs"},{"type":"extras","title":"Compression - KafkaEx","doc":"Snappy and gzip compression is supported.  Example usage for producing compressed messages:\n\n```elixir\nmessage1 = %KafkaEx.Protocol.Produce.Message{value: \"value 1\"}\nmessage2 = %KafkaEx.Protocol.Produce.Message{key: \"key 2\", value: \"value 2\"}\nmessages = [message1, message2]\n\n#snappy\nproduce_request = %KafkaEx.Protocol.Produce.Request{\n  topic: \"test_topic\",\n  partition: 0,\n  required_acks: 1,\n  compression: :snappy,\n  messages: messages}\nKafkaEx.produce(produce_request)\n\n#gzip\nproduce_request = %KafkaEx.Protocol.Produce.Request{\n  topic: \"test_topic\",\n  partition: 0,\n  required_acks: 1,\n  compression: :gzip,\n  messages: messages}\nKafkaEx.produce(produce_request)\n```\n\nCompression is handled automatically on the consuming/fetching end.","ref":"readme.html#compression"},{"type":"extras","title":"Testing - KafkaEx","doc":"It is strongly recommended to test using the Dockerized test cluster described\nbelow.  This is required for contributions to KafkaEx.\n\n**NOTE** You may have to run the test suite twice to get tests to pass.  Due to\nasynchronous issues, the test suite sometimes fails on the first try.\n\n#","ref":"readme.html#testing"},{"type":"extras","title":"Dockerized Test Cluster - KafkaEx","doc":"Testing KafkaEx requires a local SSL-enabled Kafka cluster with 3 nodes: one\nnode listening on each port 9092, 9093, and 9093.  The easiest way to do this\nis using the scripts in\nthis repository that utilize [Docker](https://www.docker.com) and\n[Docker Compose](https://www.docker.com/products/docker-compose) (both of which\nare freely available).  This is the method we use for our CI testing of\nKafkaEx.\n\nTo launch the included test cluster, run\n\n```\n./scripts/docker_up.sh\n```\n\nThe `docker_up.sh` script will attempt to determine an IP address for your\ncomputer on an active network interface.\n\nThe test cluster runs Kafka 0.11.0.1.\n\n#","ref":"readme.html#dockerized-test-cluster"},{"type":"extras","title":"Running the KafkaEx Tests - KafkaEx","doc":"The KafkaEx tests are split up using tags to handle testing multiple scenarios\nand Kafka versions.\n\n##","ref":"readme.html#running-the-kafkaex-tests"},{"type":"extras","title":"Unit tests - KafkaEx","doc":"These tests do not require a Kafka cluster to be running (see test/test_helper.exs:3 for the tags excluded when running this).\n\n```\nmix test --no-start\n```\n\n##","ref":"readme.html#unit-tests"},{"type":"extras","title":"Integration tests - KafkaEx","doc":"If you are not using the Docker test cluster, you may need to modify\n`config/config.exs` for your set up.\n\nThe full test suite requires Kafka 2.1.0+.\n\n###","ref":"readme.html#integration-tests"},{"type":"extras","title":"Kafka >= 0.9.0 - KafkaEx","doc":"The 0.9 client includes functionality that cannot be tested with older\nclusters.\n\n```\n./scripts/all_tests.sh\n```\n\n###","ref":"readme.html#kafka-0-9-0"},{"type":"extras","title":"Kafka = 0.9.0 - KafkaEx","doc":"The 0.9 client includes functionality that cannot be tested with older\nclusters.\n\n```\nmix test --include integration --include consumer_group --include server_0_p_9_p_0\n```\n\n###","ref":"readme.html#kafka-0-9-0"},{"type":"extras","title":"Kafka >= 0.8.2 and < 0.9.0 - KafkaEx","doc":"Kafka 0.8.2 introduced the consumer group API.\n\n```\nmix test --include consumer_group --include integration\n```\n\n###","ref":"readme.html#kafka-0-8-2-and-0-9-0"},{"type":"extras","title":"Kafka < 0.8.2 - KafkaEx","doc":"If your test cluster is older, the consumer group tests must be omitted.\n\n```\nmix test --include integration --include server_0_p_8_p_0\n```\n\n#","ref":"readme.html#kafka-0-8-2"},{"type":"extras","title":"Static analysis - KafkaEx","doc":"```\nmix dialyzer\n```","ref":"readme.html#static-analysis"},{"type":"extras","title":"Contributing - KafkaEx","doc":"All contributions are managed through the\n[kafkaex github repo](https://github.com/kafkaex/kafka_ex).\n\nIf you find a bug or would like to contribute, please open an\n[issue](https://github.com/kafkaex/kafka_ex/issues) or submit a pull\nrequest.  Please refer to [CONTRIBUTING.md](CONTRIBUTING.md) for our\ncontribution process.\n\nKafkaEx has a Slack channel: #kafkaex on\n[elixir-lang.slack.com](http://elixir-lang.slack.com). You can request\nan invite via [http://bit.ly/slackelixir](http://bit.ly/slackelixir).\nThe Slack channel is appropriate for quick questions or general design\ndiscussions.  The Slack discussion is archived at\n[http://slack.elixirhq.com/kafkaex](http://slack.elixirhq.com/kafkaex).","ref":"readme.html#contributing"},{"type":"extras","title":"Default snappy algorithm use snappyer package - KafkaEx","doc":"It can be changed to snappy by using this:\n\n``` elixir\nconfig :kafka_ex, snappy_module: :snappy\n```\n\nSnappy erlang nif is deprecated and will be dropped 1.0.0 release.","ref":"readme.html#default-snappy-algorithm-use-snappyer-package"},{"type":"extras","title":"Kayrock-Based KafkaEx Client","doc":"# Kayrock-Based KafkaEx Client\n\nThis document contains information on using the new\n[Kayrock](https://github.com/dantswain/kayrock)-based KafkaEx client.\n\nThe client itself is defined in the module `KafkaEx.New.Client`.  This will be\nthe primary KafkaEx client module going forward.\n\n**NOTE** In many places below we recommend using \"version N and up\".  The\nreality is that in many cases, the KafkaEx legacy API compatibility only\nsupports versions 0 to 3.  Using version 3 should generally be safe and achieve\nthe desired outcomes.  The new API will be designed to handle newer versions.\n\nContents:\n\n*   [Using the New Client](#using-the-new-client)\n*   [Common Use Case - Store Offsets In\n     Kafka](#common-use-case-store-offsets-in-kafka)\n*   [Common Use Case - Message Timestamps / New Storage Format](#common-use-case-message-timestamps-new-storage-format)","ref":"kayrock.html"},{"type":"extras","title":"Using the New Client - Kayrock-Based KafkaEx Client","doc":"To use the new client in your project, set `kafka_version: \"kayrock\"` in your\nconfig file:\n\n```\nconfig :kafka_ex,\n  kafka_version: \"kayrock\"\n```\n\nIf want to start a single client instance, supply the new client's module\nname to the `:server_impl` option of `KafkaEx.start_link_worker/2`:\n\n```\n{:ok, pid} = KafkaEx.start_link_worker(:no_name, server_impl: KafkaEx.New.Client)\n```\n\nThe new client should be totally backwards compatible with the legacy API;\nsimply use KafkaEx like you normally would.  It is also compatible with the new\nAPI defined in `KafkaEx.New.KafkaExAPI`.","ref":"kayrock.html#using-the-new-client"},{"type":"extras","title":"Common Use Case - Store Offsets In Kafka - Kayrock-Based KafkaEx Client","doc":"Offsets are stored in Kafka (instead of Zookeeper) with offset commit message\nversion 1 and up (Kafka v0.10 and up). Message version 1 also includes a\ntimestamp parameter that was dropped in version 2, so we recommend using at\nleast version 2. We often use version 3 out of convenience because that\nversion number achieves desired results with this and other Kafka API\nmessages.\n\nTo retrieve offsets committed with version 1 and up, you must also use version 1\nand up of the offset fetch request.  A safe move here is to always use the same\nversion for offset commit and offset fetch.\n\n#","ref":"kayrock.html#common-use-case-store-offsets-in-kafka"},{"type":"extras","title":"WARNING - OFFSET LOSS - Kayrock-Based KafkaEx Client","doc":"If offsets for a consumer group are stored in Zookeeper (v0 KafkaEx legacy),\nthey are unavailable using v1 and up of the offset fetch message.  This means\nthat if you have an existing KafkaEx consumer group and \"upgrade\" to the new\nversion, **you will lose offsets**.  To avoid losing offsets, you should first\nconvert the Zookeeper offsets to Kafka storage.  This can be achieved using\ncommand line tools (documentation of which is beyond the scope of this document)\nor using KafkaEx by fetching the offsets using v0 and then committing them using\nv1.  We may provide a tool for this in the future.\n\nLikewise, once you store offsets in Kafka, they cannot be fetched using v0 of\nthe offset fetch message.  If you need to \"roll back\" storage from Kafka to\nZookeeper, you will need to convert the offsets first.\n\n#","ref":"kayrock.html#warning-offset-loss"},{"type":"extras","title":"Examples - Kayrock-Based KafkaEx Client","doc":"`KafkaEx.offset_commit/2` and `KafkaEx.offset_fetch/2` support setting the api\nversion via the `api_version` field of their corresponding requests structs:\n\n```\nalias KafkaEx.Protocol.OffsetCommit\n\n# commit offsets using kafka storage\nKafkaEx.offset_commit(client, %OffsetCommit.Request{\n        consumer_group: consumer_group,\n        topic: topic,\n        partition: 0,\n        offset: offset,\n        api_version: 3\n      })\n\n# fetch an offset stored in kafka\n[resp] = KafkaEx.offset_fetch(client, %OffsetFetch.Request{\n        topic: topic,\n        consumer_group: consumer_group,\n        partition: 0,\n        api_version: 3\n      })\n%KafkaEx.Protocol.OffsetFetch.Response{partitions: [%{offset: offset}]} = resp\n```\n\nWhen using `KafkaEx.fetch/3` with `auto_commit: true`, you can specify the\n`offset_commit_api_version` option to control how offsets are stored:\n\n```\n# store auto-committed offsets in kafka\nKafkaEx.fetch(topic, partition, auto_commit: true, offset_commit_api_version: 3)\n```\n\nWhen using `KafkaEx.ConsumerGroup`, you can control offset storage using the\n`api_versions` option:\n\n```\n# use kafka offset storage with consumer groups\n# NOTE you must use compatible version for offset_fetch and offset_commit \n#   using the same value for both should be safe\nKafkaEx.ConsumerGroup.start_link(\n    MyConsumer,\n    consumer_group_name,\n    [topic],\n    api_versions: %{offset_fetch: 3, offset_commit: 3}\n)\n```","ref":"kayrock.html#examples"},{"type":"extras","title":"Common Use Case - Message Timestamps / New Storage Format - Kayrock-Based KafkaEx Client","doc":"Message timestamps and the [new message storage\nformat](https://kafka.apache.org/documentation/#recordbatch) go hand-in-hand\nbecause they both require setting the message versions for produce and fetch.\n\nTimestamps were added in v1 of the produce/fetch messages, but the storage\nformat was replaced in v2 (around Kafka v0.10), so we recommend using 2 and up\n(3 is safe).\n\n#","ref":"kayrock.html#common-use-case-message-timestamps-new-storage-format"},{"type":"extras","title":"WARNING - Broker Performance - Kayrock-Based KafkaEx Client","doc":"Check with your system administrator.  If the broker is configured to use the\nnew message format (v2 and up), producing or requesting messages with old\nformats (v0 and v1) can lead to significant load on the brokers because they\nwill need to convert messages between versions for each request.  If you have a\nrelatively modern version of Kafka, we recommend using version 3 for both\nmessages.\n\n#","ref":"kayrock.html#warning-broker-performance"},{"type":"extras","title":"Examples - Kayrock-Based KafkaEx Client","doc":"Whenever we use produce API version `>= 2`, the new message format is used automatically.\n\nTo publish a message with a timestamp:\n\n```\n{:ok, offset} =\n  KafkaEx.produce(\n    topic,\n    0,\n    msg,\n    worker_name: client,\n    required_acks: 1,\n    timestamp: 12345,\n    api_version: 3\n  )\n\nfetch_responses =\n  KafkaEx.fetch(topic, 0,\n    offset: offset,\n    auto_commit: false,\n    worker_name: client,\n    api_version: 3\n  )\n\n[fetch_response | _] = fetch_responses\n[partition_response | _] = fetch_response.partitions\nmessage = List.last(partition_response.message_set)\n\nmessage.value # => msg\nmessage.offset # => offste\nmessage.timestamp # => 12345\n```\n\nIf a topic has the `message.timestamp.type` setting set to `LogAppendTime`, then\ntimestamps will be populated automatically when a produced message is received\nby the broker and appended to the log.\n\n```\nfetch_responses =\n  KafkaEx.fetch(topic_with_log_append_timestamps, 0,\n    offset: offset,\n    auto_commit: false,\n    worker_name: client,\n    api_version: 3\n  )\n\n[fetch_response | _] = fetch_responses\n[partition_response | _] = fetch_response.partitions\nmessage = List.last(partition_response.message_set)\n\nmessage.timestamp # => log append timestamp in milliseconds\n```\n\nNote that the `KafkaEx.ConsumerGroup` `api_versions` option also supports\nsetting a version for `fetch`:\n\n```\n# use new record batch format AND kafka offset storage with consumer groups\nKafkaEx.ConsumerGroup.start_link(\n    MyConsumer,\n    consumer_group_name,\n    [topic],\n    api_versions: %{offset_fetch: 3, offset_commit: 3, fetch: 3}\n)\n```","ref":"kayrock.html#examples"},{"type":"extras","title":"KafkaEx 1.0 API","doc":"# KafkaEx 1.0 API\n\nThis document describes the design approach and gives an overview of thew new\nclient API.  The API itself is documented in `KafkaEx.New.KafkaExAPI`.  The\ncurrent plan is for `KafkaEx.New.KafkaExAPI` to replace `KafkaEx` in the v1.0\nrelease.\n\nThe new API is designed to continue to provide a useful Kafka client API\nforemost, to address some of the limitations and inconveniences of the\nexisting API (both in terms of usage and maintenance).  A central goal of the\nnew API is to allow us to support new Kafka features more rapidly than in the\npast.","ref":"new_api.html"},{"type":"extras","title":"Status - KafkaEx 1.0 API","doc":"The new API is still in very early stages of development.  We will try to keep\nthis section up-to-date with respect to what features have been implemented.\n`KafkaEx.New.KafkaExAPI` is the source of truth for this summary.\n\nFeatures implemented:\n\n*   Get latest offset for a partition as `{:ok, offset}` or `{:error, error_code}`\n    (no more fishing through the response structs).\n*   Get metadata for an arbitrary list of topics","ref":"new_api.html#status"},{"type":"extras","title":"Major Differences from the Legacy API - KafkaEx 1.0 API","doc":"*   There is currently no supervisor for clients.  It is assumed that the user\n    will manage these when not used in a consumer group.  (This does not apply to\n    clients started via the legacy `create_worker` API, which are started under the standard\n    supervision tree.)\n*   The client does not automatically fetch metadata for all topics as this can\n    lead to timeouts on large clusters.  There should be no observable impact here\n    because the client fetches metadata for specific topics on-demand.\n*   A client is no longer \"attached\" to a specific consumer group.  In the legacy\n    implementation this was a consequence of the way autocommit was handled.","ref":"new_api.html#major-differences-from-the-legacy-api"},{"type":"extras","title":"Design Philosophy - KafkaEx 1.0 API","doc":"Two main design principles in the new client are driven by factors that made\nmaintenance of the legacy API difficult:\n\n1.  Delegate and generalize API message version handling\n\n    Kafka API message serialization and deserialization has been externalized to\n    a library ([Kayrock](https://github.com/dantswain/kayrock)) that can easily\n    support new message formats as they are released. Kayrock exposes a generic\n    serialize/deserialize functionality, which means we do not have to write code\n    to handle specific versions of specific messages at a low level in KafkaEx.\n\n\n2.  Separation of connection state management and API logic\n\n    As much as possible, we avoid putting API logic inside the client GenServer.\n    Instead, we write functions that form Kayrock request structs based on user\n    input, use `KafkaEx.New.Client.send_request/3` to perform communication with\n    the cluster, and then act accordingly based on the response.\n\n#","ref":"new_api.html#design-philosophy"},{"type":"extras","title":"Example: Fetching Kafka Config Values - KafkaEx 1.0 API","doc":"Suppose that we wanted to implement a function to retrieve Kafka broker\nconfig settings. This is the DescribeConfig API and corresponds to the\n`Kayrock.DescribeConfig` namespace. After some research (and trial-and-error\nusing iex and functions like the ones below), we find that to fetch broker\nsettings, we find that we need to set `resource_type` to 4 and\n`resource_name` to the string representation of the broker's id number (e.g.,\n`1 => \"1\"`):\n\n```\n# List all config values for node 1 of the cluster\n{:ok, pid} = KafkaEx.start_link_worker(:no_name, server_impl: KafkaEx.New.Client)\n\n# resource type 4 is 'broker'\nreq = %Kayrock.DescribeConfigs.V0.Request{\n    resources: [%{resource_type: 4, resource_name: \"1\", config_names: nil}]\n}\n\nKafkaEx.New.Client.send_request(pid, req, KafkaEx.New.NodeSelector.node_id(1))  \n\n{:ok,\n %Kayrock.DescribeConfigs.V0.Response{\n   correlation_id: 15,\n   resources: [\n     %{\n       config_entries: [\n         %{\n           config_name: \"advertised.host.name\",\n           config_value: nil,\n           is_default: 1,\n           is_sensitive: 0,\n           read_only: 1\n         },\n         %{\n           config_name: \"log.cleaner.min.compaction.lag.ms\",\n           config_value: \"0\",\n           is_default: 1,\n           is_sensitive: 0,\n           read_only: 1\n         },\n\n         ...\n\n       ]\n     }\n   ]\n }}\n```\n\nFrom the above, we could feasibly write a convenience function to fetch broker\nconfigs and doing so would not require us to modify the client code at all.\nFurthermore, as the DescribeConfigs API changes over time, we can easily support\nit by changing only that function.  The goal of the new API is that most simple\nfunctionality can be implemented simply by writing such a function:\n\n```\nalias KafkaEx.New.Client\nalias KafkaEx.New.NodeSelector\n\ndef get_broker_config_values(client, config_names, broker_id) do\n  request = %Kayrock.DescribeConfigs.V0.Request{\n      resources: [%{\n          resource_type: 4,\n          resource_name: \"#{broker_id}\",\n          config_names: config_names\n      }]\n  }\n\n  # note this requires no changes to the client itself\n  {:ok, %{\n      resources: [\n          %{\n              error_code: 0,\n              config_entries: config_entries\n          }\n      ]\n  }} = Client.send_request(client, request, NodeSelector.node_id(broker_id))\n\n  config_entries\nend\n```\n\nThis implementation only supports version 0 of the DescribeConfigs API.  We can\nachieve some level of forward compatibility by adding an `opts` keyword list\nand some code to handle `api_version` in the opts:\n\n```\ndef get_broker_config_values(client, config_names, broker_id, opts \\\\ []) do\n  api_version = Keyword.get(opts, :api_version, 0)\n  # a setting in v1+\n  include_synonyms = Keyword.get(opts, :include_synonyms, false)\n\n  request = Kayrock.DescribeConfigs.get_request_struct(api_version)\n\n  request = \n    if api_version > 1 do\n      %{request | include_synonyms: include_synonyms}\n    else\n      request\n    end\n\n  # rest is the same\nend\n```","ref":"new_api.html#example-fetching-kafka-config-values"},{"type":"extras","title":"Contributing","doc":"# Contributing\n\n- [Fork](https://github.com/kafkaex/kafka_ex/fork), then clone the repo: `git clone git@github.com:your-username/kafka_ex.git`\n- Create a feature branch: `git checkout -b feature_branch`\n- Make your changes\n- Make sure the tests all pass with the dockerized test cluster.  See the \"Testing\" section of the README.\n```\nmix test --include integration --include consumer_group --include server_0_p_9_p_0\n```\n- Make sure the unit tests pass: `mix test --no-start`\n- Make sure the integration tests pass: `mix test --only integration`\n- Make sure the consumer group tests pass: `mix test --only consumer_group`\n- Make sure dialyzer returns clean: `mix dialyzer` *See below*\n- Push your feature branch: `git push origin feature_branch`\n- Submit a pull request with your feature branch\n\nThanks!\n\n*Dialyzer note* You need Elixir 1.3.2+ to run `mix dialyzer`.  You may get some\nfalse positives on Erlang 18.  `mix dialyzer` is known to return clean on\nElixir 1.3.4 with Erlang 19.2.","ref":"contributing.html"}],"content_type":"text/markdown"}